package org.processmining.plugins.fuzzymap;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.plugin.events.Logger.MessageLevel;
import org.processmining.models.graphbased.directed.patterntree.PatternTree;
import org.processmining.plugins.abstractions.PatternAbstractionFrame;
import org.processmining.plugins.fuzzymap.ui.FuzzyMapToolkitPanel;
import org.processmining.plugins.ptnBasedPreprocess.util.PatternAbstractionFrameUtil;

/**
 * Visualization support for Fuzzy Map. *
 * 
 * @author Jiafei Li (jiafei@jlu.edu.cn)
 * 
 */
@Plugin(name = "Fuzzy Map visualization", returnLabels = { "Visualized Abstraction Tree and Fuzzy Map" }, returnTypes = { JComponent.class }, parameterLabels = {
		"Pattern Tree", "FilteredPatternAlphabetSequenceMap", "AbstractionSequenceMap", "log", "MiningParameters" }, userAccessible = false)
@Visualizer
public class FuzzyMapVisualization {
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(UIPluginContext context, PatternTree patternTree) {
		System.out.println("Fuzzy Map miner Visualizer Start!!!");
		System.out.println("The Fuzzy Map Visualization context is " + context.toString());
		try {
			System.out.println("Start to get FuzzyMapConnection!");
			ConnectionManager cm = context.getConnectionManager();
			FuzzyMapConnection fmConn = cm.getFirstConnection(FuzzyMapConnection.class, context);
			System.out.println("have got the  FuzzyMapConnection");

			// connection found. Create all necessary component to instantiate inactive visualization panel
			PatternAbstractionFrameUtil ptnAbsFrm = fmConn.getObjectWithRole(FuzzyMapConnection.PATTERN_ABSTRACTION_FRAME);
			Set<Set<String>> filterPatternAlphabetSet = fmConn
					.getObjectWithRole(FuzzyMapConnection.FILTERED_PATTERN_ALPHABETSET);
			XLog log = fmConn.getObjectWithRole(FuzzyMapConnection.LOG);
			ArrayList<Object> miningParas = fmConn.getObjectWithRole(FuzzyMapConnection.MINING_PARAMETERS);
			return getVisualizationPanel(context, patternTree, ptnAbsFrm, filterPatternAlphabetSet, log, miningParas);
		} catch (ConnectionCannotBeObtained e) {
			// No connections available
			context.log("Connection is not exist", MessageLevel.DEBUG);
			getErrorMessagePanel("No connections available");
			return null;

		}

	}

	/**
	 * This method only invoked after all connection checking is performed
	 * 
	 * @param context
	 * @param graph
	 * @param log
	 * @param metrics
	 * @param numberOfNodes
	 * @return
	 */
	private JComponent getVisualizationPanel(UIPluginContext context, PatternTree patternTree,
			PatternAbstractionFrameUtil ptnAbsFrm, Set<Set<String>> filterPatternAlphabetSet, XLog log,
			List<Object> miningParas) {
		System.out.println("Start to Get the Abstraction Tree and Fuzzy Map VisualizationPanel!!!");
		FuzzyMapToolkitPanel fmtPanel = new FuzzyMapToolkitPanel(context,patternTree, ptnAbsFrm,
				filterPatternAlphabetSet, log, miningParas);
		// return result
		return fmtPanel;
	}

	/**
	 * return error message panel
	 * 
	 */
	private JComponent getErrorMessagePanel(String message) {
		JLabel errorMessage = new JLabel();
		errorMessage.setText(message);

		JPanel panel = new JPanel();
		panel.add(errorMessage);

		return panel;
	}
}
