package org.processmining.plugins.fuzzymap.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.out.XesXmlGZIPSerializer;
import org.jgraph.event.GraphSelectionEvent;
import org.jgraph.event.GraphSelectionListener;
import org.jgraph.graph.GraphSelectionModel;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.events.Logger.MessageLevel;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.fuzzymodel.attenuation.Attenuation;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.MetricsRepository;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.binary.BinaryDerivateMetric;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.binary.BinaryLogMetric;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.unary.UnaryMetric;
import org.processmining.models.graphbased.directed.jgraph.JGraphVisualizationPanel;
import org.processmining.models.graphbased.directed.jgraph.ProMJGraph;
import org.processmining.models.graphbased.directed.jgraph.ProMJGraphVisualizer;
import org.processmining.models.graphbased.directed.jgraph.elements.ProMGraphCell;
import org.processmining.models.graphbased.directed.patterntree.PatternTree;
import org.processmining.models.graphbased.directed.patterntree.PatternTreeEdge;
import org.processmining.models.graphbased.directed.patterntree.PatternTreeNode;
import org.processmining.plugins.abstractions.PatternAbstractionFrame;
import org.processmining.plugins.abstractions.swingx.ErrorDialog;
import org.processmining.plugins.abstractions.util.EquivalenceClass;
import org.processmining.plugins.abstractions.util.UkkonenSuffixTree;
import org.processmining.plugins.fuzzymap.FuzzyBusinessProcessMap;
import org.processmining.plugins.fuzzymap.util.FuzzyMapUtil;
import org.processmining.plugins.fuzzymodel.miner.edit.FuzzyGraphEditorPanel;
import org.processmining.plugins.fuzzymodel.miner.ui.FuzzyModelInformationPanel;
import org.processmining.plugins.ptnBasedPreprocess.PatternTreeImpl;
import org.processmining.plugins.ptnBasedPreprocess.util.PatternAbstractionFrameUtil;

import com.fluxicon.slickerbox.components.HeaderBar;
import com.fluxicon.slickerbox.components.SlickerButton;
import com.fluxicon.slickerbox.ui.SlickerRadioButtonUI;
import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.hierarchical.JGraphHierarchicalLayout;

public class AbstractionTreePanel extends JPanel {

	/**
	 * 
	 */
	protected static final boolean DEBUG = true;
	protected static final int NOT_USE_YET = 0;
	protected static final int SINGLE_PATTERN_MODE = 1;
	protected static final int ABSTRACTION_MODE = 2;
	private static final long serialVersionUID = -3751299572607459175L;
	protected Color background = new Color(100, 100, 100);
	protected PatternTree patternTree;
	protected Set<Set<String>> filterPatternAlphabetSet;
	//keep the map between the new pattern name and its encoded alphabet in current pattern graph
	protected HashMap<String, Set<Set<String>>> newOldPatternNameMap;
	//Map between the pattern (encoded) and the  count of its total occurrence in the non-overlapping condition
	protected HashMap<String, Double> patternTotalNonOverlapAlphabetCountMap;
	Map<TreeSet<String>, Integer> actualNonOverlapAlphabetConservednessMetricMap;
	//Map between the pattern (encoded) and the count of its isolation occurrence in the non-overlapping condition
	protected Map<TreeSet<String>, Integer> patternIsolationNonOverlapAlphabetCountMap;
	protected HashMap<String, List<String>> ptnLabelDecodedActivityNameMap;
	protected Map<Set<String>, TreeSet<String>> patAlphabetSeqMap;
	protected Map<Set<String>, TreeSet<String>> absSeqMap;
	protected Map<TreeSet<String>, Integer> actualNonOverlapAlphabetCountMap;
	//Map between the new label of each selected pattern and its alphabet
	protected HashMap<String, Set<Set<String>>> selectedPatternNameAlphabetMap;
	//Map between the new label of each selected pattern and its possible sequences
	protected HashMap<String, Set<String>> selectedPatternNameSequenceMap;
	//Map between the parent pattern and its offspring patterns
	protected Map<Set<String>, Set<Set<String>>> parentOffspringsPatternMap;
	//Map between the parent pattern and its offspring pattern nodes
	protected Map<PatternTreeNode, Set<PatternTreeNode>> parentOffspringNodesMap;
	//the set of the current selected pattern to use
	private final Set<Set<String>> oldLabelsOfSelectedPatterns;
	//map between selected pattern alphabet and its new pattern label
	private final Map<Set<String>, String> selectedPatternAlphabetNewLabel;

	protected List<String> encodedTraceList;
	protected PluginContext context;
	protected XLog log;
	protected JPanel wholePanel;
	protected JGraphVisualizationPanel graphPanel;
	protected JSplitPane patternEditPanel;
	protected JPanel editPanel;
	protected JPanel desiredPatternPanel;
	protected List<Object> miningParameters = new ArrayList<Object>();
	protected final JPanel parentPanel;
	protected JTextField nameField;
	protected JLabel nameLabel;
	protected JLabel activityLabel;
	protected JList activityList;
	protected DefaultListModel activityListModel;
	protected JRadioButton notToUseRadioButton;
	protected JRadioButton desireNodeRadioButton;
	protected JRadioButton desireSubGraphRadioButton;
	protected JLabel patternLabel;
	protected JList patternList;
	protected DefaultListModel patternListModel;
	protected Map<String, String> charActivityMap;
	protected List<String> encodeTraceList;
	protected PatternAbstractionFrameUtil ptnAbsFrm;
	protected PatternTreeNode selectedPatternNode;
	protected String beforeModifiedPatterLabel;
	//private Set<String> newLabelsOfSelectedPattern;
	static int encodingLength;
	protected Boolean isThereDuplicatePattern = false;
	protected String orinalLabelOfCurrentPattern;
	protected MetricsRepository metrics;
	protected Attenuation attenuation;
	protected Integer maxRelDistance;

	//private LogRelations originLogRelations; // log relations

	public AbstractionTreePanel(UIPluginContext context, JPanel parentPanel, PatternTree patternTree,
			PatternAbstractionFrameUtil ptnAbsFrame, Set<Set<String>> filterPatternAlphabetSet, XLog log,
			List<Object> miningParas) {
		this.context = context;
		this.parentPanel = parentPanel;
		this.log = log;
		this.patternTree = patternTree;
		//	this.renamedPatternTree = originalPatternTree;
		miningParameters = miningParas;
		metrics = (MetricsRepository) miningParameters.get(0);
		attenuation = (Attenuation) miningParameters.get(1);
		maxRelDistance = (Integer) miningParameters.get(2);
		ptnAbsFrm = ptnAbsFrame;
		//ptnAbsFrm.findRepeatPatternsInLog();
		this.filterPatternAlphabetSet = filterPatternAlphabetSet;
		actualNonOverlapAlphabetConservednessMetricMap = ptnAbsFrm.getActualNonOverlapAlphabetConservednessMetricMap();
		//get filtered pattern and its alphabet sequence set map
		patAlphabetSeqMap = ptnAbsFrm.getFilterPatternAlphabetSequenceSetMap(filterPatternAlphabetSet);
		//get abstraction and its alphabet sequence set map
		absSeqMap = ptnAbsFrm.getAbstractionAlphabetSequenceSetMap(filterPatternAlphabetSet);
		actualNonOverlapAlphabetCountMap = ptnAbsFrm.getNonOverlapAlphabetCountMap();
		encodingLength = ptnAbsFrm.getEncodingLength();
		charActivityMap = new HashMap<String, String>();
		charActivityMap = ptnAbsFrm.getCharActivityMap();
		encodeTraceList = new ArrayList<String>();
		encodeTraceList = ptnAbsFrm.getEncodedTraceList();
		parentOffspringsPatternMap = new HashMap<Set<String>, Set<Set<String>>>();
		parentOffspringNodesMap = new HashMap<PatternTreeNode, Set<PatternTreeNode>>();
		oldLabelsOfSelectedPatterns = new HashSet<Set<String>>();
		selectedPatternNameAlphabetMap = new HashMap<String, Set<Set<String>>>();
		selectedPatternAlphabetNewLabel = new HashMap<Set<String>, String>();
		//	this.originLogRelations = new BasicLogRelations(this.log);
		setupUI();
	}

	public void setupUI() {

		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder());
		//set up the pattern edit panel
		setupEditPanel();
		//set up the desired pattern panel
		setupDesiredPatternPanel();
		//assemble patternEditPanel
		JScrollPane editScrollPane = new JScrollPane();
		editScrollPane.setViewportView(editPanel);
		editScrollPane.setOpaque(false);
		//Dimension d1 = editPanel.getPreferredSize();
		//	Dimension d2 = editPanel.getMinimumSize();
		//System.out.println("The preferred size of EditPanel is " + d1.width + " , " + d1.height);
		//System.out.println("The min size of EditPanel is " + d2.width + " , " + d2.height);
		editScrollPane.setPreferredSize(new Dimension(300, 600));
		//	editScrollPane.setSize(new Dimension(350,600));
		JScrollPane desiredPatternScrollPane = new JScrollPane();
		desiredPatternScrollPane.setViewportView(desiredPatternPanel);
		desiredPatternScrollPane.setOpaque(false);
		//	Dimension d3 = desiredPatternPanel.getPreferredSize();
		//Dimension d4 = desiredPatternPanel.getMinimumSize();
		//System.out.println("The preferred size of desiredPatternPanel is " + d3.width + " , " + d3.height);
		//	System.out.println("The min size of desiredPatternPanel is " + d4.width + " , " + d4.height);
		desiredPatternScrollPane.setPreferredSize(new Dimension(300, 400));
		//desiredPatternScrollPane.setMinimumSize(new Dimension(350,400));
		patternEditPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT, editScrollPane, desiredPatternScrollPane);
		//	Dimension splitPanelSize = new Dimension(300, 1000);
		//patternEditPanel.setPreferredSize(splitPanelSize);
		patternEditPanel.setResizeWeight(0.5);
		//	patternEditPanel.setMinimumSize(splitPanelSize);
		//patternEditPanel.setMaximumSize(splitPanelSize);
		//patternEditPanel.setDividerLocation(0.5);
		//rename the node in the original pattern tree
		renamePatternTreeNodes();

		//set the pattern tree panel
		graphPanel = ProMJGraphVisualizer.visualizeGraph(patternTree);
		ProMJGraph jgraph = graphPanel.getGraph();
		//change the layout of the graph
		JGraphHierarchicalLayout layout = new JGraphHierarchicalLayout();
		layout.setOrientation(SwingConstants.WEST);
		JGraphFacade facade = new JGraphFacade(jgraph);

		facade.setOrdered(true);
		facade.setEdgePromotion(true);
		facade.setIgnoresCellsInGroups(false);
		facade.setIgnoresHiddenCells(false);
		facade.setIgnoresUnconnectedCells(false);
		facade.setDirected(false);
		facade.resetControlPoints();
		facade.run(layout, false);

		//fixParallelEdges(facade, 15);

		Map<?, ?> nested = facade.createNestedMap(true, false);

		jgraph.getGraphLayoutCache().edit(nested);
		// graph.setLayedOut(true);

		//graphPanel.getGraph().getGraphLayoutCache().
		//graphPanel.getGraph().getLayout().
		/*
		 * ProMJGraph patternJGraph = graphPanel.getGraph(); patternJGraph
		 */
		//patternTree.getAttributeMap().put(AttributeMap.PREF_ORIENTATION, SwingConstants.NORTH);
		//	patternTree.signalViews();
		addAncestorListener(new AncestorListener() {
			public void ancestorAdded(AncestorEvent event) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						updateGraphLayout();
					}
				});
			}

			public void ancestorMoved(AncestorEvent event) {
				// ignore
			}

			public void ancestorRemoved(AncestorEvent event) {
				// ignore
			}
		});

		//graphPanel.setBounds(this.getX(), this.getY(), (int)(this.getWidth() * 0.7), (int)(this.getHeight() * 0.7));
		//wholePanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,graphPanel,patternEditPanel);
		wholePanel = new JPanel();
		wholePanel.setLayout(new BorderLayout());
		wholePanel.setBorder(BorderFactory.createEmptyBorder());
		wholePanel.add(graphPanel, BorderLayout.CENTER);
		wholePanel.add(patternEditPanel, BorderLayout.EAST);
		//assemble in the panel
		HeaderBar header = new HeaderBar("Pattern Tree (mined from " + XConceptExtension.instance().extractName(log)
				+ ")");
		header.setHeight(40);
		this.add(header, BorderLayout.NORTH);
		this.add(wholePanel, BorderLayout.CENTER);
	}

	//rename the node in the pattern tree
	public void renamePatternTreeNodes() {
		//the map between new name of a pattern node and its old one
		//newOldPatternNameMap = new HashMap<String, Set<String>>();
		newOldPatternNameMap = new HashMap<String, Set<Set<String>>>();
		ptnLabelDecodedActivityNameMap = new HashMap<String, List<String>>();
		patternTotalNonOverlapAlphabetCountMap = new HashMap<String, Double>();
		//prepare the count of each isolation pattern
		Set<String> actualPatternSet = new HashSet<String>();
		for (Set<String> alphabet : filterPatternAlphabetSet) {
			actualPatternSet.addAll(patAlphabetSeqMap.get(alphabet));
		}
		//using the longer pattern first
		ptnAbsFrm.findNonoverlapPatternCountAcrossAllPatterns(false);
		patternIsolationNonOverlapAlphabetCountMap = ptnAbsFrm.getActualStandAloneAlphabetCountMap();
		Set<PatternTreeNode> nodes = patternTree.getNodes();
		Double alphabetTotalCount;
		Double alphabetStandaloneCount;
		Double alphabetConservednessCount;
		//if there were same subsequence of two different patterns, 
		//modify the 2nd occurred pattern name according to the identifier .eg. "Repair","Repair1","Repair2",etc
		Map<String, Integer> patternNameIdentifierMap = new HashMap<String, Integer>();
		for (PatternTreeNode node : nodes) {
			//get the total occurrence count of this pattern 
			alphabetTotalCount = new Double(actualNonOverlapAlphabetCountMap.get(node.getLabelAlphabet()));
			if (patternIsolationNonOverlapAlphabetCountMap.containsKey(node.getLabelAlphabet())) {
				alphabetStandaloneCount = new Double(patternIsolationNonOverlapAlphabetCountMap.get(node
						.getLabelAlphabet()));
			} else {
				alphabetStandaloneCount = new Double(0.0);
			}

			if (actualNonOverlapAlphabetConservednessMetricMap.containsKey(node.getLabelAlphabet())) {
				alphabetConservednessCount = new Double(actualNonOverlapAlphabetConservednessMetricMap.get(node
						.getLabelAlphabet()));
			} else {
				alphabetConservednessCount = 100.0;
			}

			List<String> decodedActNames = new ArrayList<String>();
			String nodeLabel = node.getLabel();
			Set<String> nodeLabelAlphabet = node.getLabelAlphabet();
			Set<Set<String>> patternAlphabetSet = new HashSet<Set<String>>();
			patternAlphabetSet.add(nodeLabelAlphabet);
			patternTotalNonOverlapAlphabetCountMap.put(nodeLabel, alphabetTotalCount);
			Object[] activityLabels = FuzzyMapUtil.getActivityLabelsInPattern(node, encodingLength).toArray();
			if (activityLabels.length == 1) {
				//decode the node name in the pattern tree
				String decodeActivityName = charActivityMap.get(activityLabels[0]);
				//the new pattern name is like " event name - event type". eg."abc-start"
				//	newOldPatternNameMap.put(decodeActivityName, nodeLabelAlphabet);
				newOldPatternNameMap.put(decodeActivityName, patternAlphabetSet);
				/*
				 * int delimIdx = decodeActivityName.indexOf("-"); String
				 * singleActivityName = decodeActivityName.substring(0,
				 * delimIdx);
				 */
				node.setNameToEdit(decodeActivityName);
				String newNodeLabel;
				//newNodeLabel = FuzzyMapUtil.getSingleActivityNodeLabel(decodeActivityName, "-");

				//add node metrics to the label
				newNodeLabel = "<html>" + decodeActivityName + "<br>" + alphabetTotalCount.toString() + "/"
						+ alphabetStandaloneCount.toString() + "/" + alphabetConservednessCount + "</html>";
				node.setLabel(newNodeLabel);
				//	newOldPatternNameMap.put(newNodeLabel, nodeLabel);
				decodedActNames.add(decodeActivityName);
			} else {
				//set the default name to the pattern node according to the longest common sequence between its activities
				Object[] decodedActivityLabels = new Object[activityLabels.length];
				int indexOfDelimiter = 0;
				String wholeLabel;
				//get the original event name of each activity in a pattern
				for (int i = 0; i < activityLabels.length; i++) {
					wholeLabel = charActivityMap.get(activityLabels[i]);
					decodedActNames.add(wholeLabel);
					indexOfDelimiter = wholeLabel.indexOf("-");
					//get the event name only
					decodedActivityLabels[i] = wholeLabel.substring(0, indexOfDelimiter);
				}
				//String defaultPatternName = getDefaultPatternName(decodedActivityLabels);
				String defaultPatternName = getAbstractionName(decodedActivityLabels);

				String originalDefaultPatternName = defaultPatternName;
				if (!newOldPatternNameMap.containsKey(defaultPatternName)) {
					//	newOldPatternNameMap.put(defaultPatternName, nodeLabel);
					patternNameIdentifierMap.put(defaultPatternName, 1);
				} else {
					Integer identifier = patternNameIdentifierMap.get(defaultPatternName);
					identifier++;
					defaultPatternName = defaultPatternName + " " + identifier.toString();
					//		newOldPatternNameMap.put(defaultPatternName, nodeLabel);
					patternNameIdentifierMap.put(originalDefaultPatternName, identifier);
				}
				//the defaultPattern Name is like "[abc], [abc] 1,[abc] 2,..."
				//	newOldPatternNameMap.put(defaultPatternName, nodeLabelAlphabet);
				newOldPatternNameMap.put(defaultPatternName, patternAlphabetSet);
				node.setNameToEdit(defaultPatternName);
				//	System.out.println("The pattern name of '" + nodeLabel + " ' for Editing is " + defaultPatternName);
				defaultPatternName = "<html>" + defaultPatternName + "<br>" + alphabetTotalCount.toString() + "/"
						+ alphabetStandaloneCount.toString() + "/" + alphabetConservednessCount + "</html>";
				node.setLabel(defaultPatternName);
			}
			ptnLabelDecodedActivityNameMap.put(nodeLabel, decodedActNames);
		}

		//set the contribution metrics of each edge
		Set<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> edges = patternTree.getEdges();
		for (PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode> edge : edges) {
			PatternTreeNode source = edge.getSource();
			PatternTreeNode target = edge.getTarget();
			Set<Set<String>> oldSourceLabelSet = newOldPatternNameMap.get(source.getNameToEdit());
			Set<Set<String>> oldTargetLabelSet = newOldPatternNameMap.get(target.getNameToEdit());
			Set<String> oldSourceLabel = oldSourceLabelSet.iterator().next();
			Set<String> oldTargetLabel = oldTargetLabelSet.iterator().next();
			Double sourceTotalCount = patternTotalNonOverlapAlphabetCountMap.get(oldSourceLabel.toString());
			Double targetTotalCount = patternTotalNonOverlapAlphabetCountMap.get(oldTargetLabel.toString());
			Double contribution = targetTotalCount / sourceTotalCount;
			String contributionLabel = FuzzyMapUtil.format(contribution);
			setEdgeLabel(edge, contributionLabel);
		}
	}

	private void setEdgeLabel(PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode> edge,
			String edgeLabel) {
		edge.getAttributeMap().put(AttributeMap.LABEL, edgeLabel);
		edge.getAttributeMap().put(AttributeMap.SHOWLABEL, true);
		edge.getAttributeMap().put(AttributeMap.LABELCOLOR, Color.red);
		edge.getAttributeMap().put(AttributeMap.LABELALONGEDGE, true);
	}

	public String getAbstractionName(Object[] decodedActivityObject) {
		String[] delimiterArray = { "~", "!", "@", "#", "%", "^", "&", "*" };
		String abstractionName = "";
		int index = 0;
		String defaultAbstractionName = "[" + (String) decodedActivityObject[0] + "]";
		StringBuilder strBuilder = new StringBuilder();

		List<String> decodedActivitySet = new ArrayList<String>();
		for (Object decodedActivity : decodedActivityObject) {
			decodedActivitySet.add((String) decodedActivity);
			//	System.out.println("In getAbstractionName: " + decodedActivitySet);
		}

		for (String decodedActivity : decodedActivitySet) {
			//			decodedActivity = decodedActivity.replaceAll("start", "").replaceAll("complete", "");
			if (index < decodedActivitySet.size() - 1) {
				strBuilder.append(decodedActivity).append(delimiterArray[index % delimiterArray.length]);
			} else {
				strBuilder.append(decodedActivity);
			}
			index++;
		}
		UkkonenSuffixTree suffixTree = new UkkonenSuffixTree(1, strBuilder.toString());
		suffixTree.findLeftDiverseNodes();
		Set<String> maximalRepeatSet = suffixTree.getMaximalRepeats();
		Set<String> filteredMaximalRepeatSet = new HashSet<String>();
		if (maximalRepeatSet != null) {
			for (String repeat : maximalRepeatSet) {
				if (repeat.length() >= 4) {
					filteredMaximalRepeatSet.add(repeat);
				}
			}
		}
		if (filteredMaximalRepeatSet.size() == 0) {
			abstractionName = defaultAbstractionName;
		} else {
			abstractionName = filteredMaximalRepeatSet.toString();
		}
		return abstractionName;
	}

	/*
	 * get the default name to the pattern node, according to the longest common
	 * sequence between its activities
	 */
	public String getDefaultPatternName(Object[] activityLabels) {
		String defaultPatternName = "";
		String commonSubString = "";
		int maxLengthOfComSubString = 0;
		for (int i = 0; i < activityLabels.length; i++) {
			String current = (String) activityLabels[i];
			for (int j = i + 1; j < activityLabels.length; j++) {
				String another = (String) activityLabels[j];
				commonSubString = FuzzyMapUtil.LongestCommonSubsequence(current, another);
				System.out.println("The LCS of '" + current + "' and '" + another + " is '" + commonSubString + " '");
				if (commonSubString.length() > maxLengthOfComSubString) {
					defaultPatternName = commonSubString;
					maxLengthOfComSubString = commonSubString.length();
				}
			}
		}
		return defaultPatternName;
	}

	public ProMJGraph updateGraphLayout() {
		//	GGGraph gGraph = new GGAdapter(graph);
		ProMJGraph paintGraph = graphPanel.getGraph();
		installGraphListeners(paintGraph);
		//	this.graphView.setGraph(gGraph);
		revalidate();
		this.repaint();
		return paintGraph;
	}

	protected void installGraphListeners(ProMJGraph graph) {

		// set the action whenever a node in the graph is selected
		GraphSelectionModel model = graph.getSelectionModel();
		model.setSelectionMode(GraphSelectionModel.SINGLE_GRAPH_SELECTION);
		model.addGraphSelectionListener(new GraphSelectionListener() {

			public void valueChanged(GraphSelectionEvent evt) {
				for (Object cell : evt.getCells()) {
					if (evt.isAddedCell(cell)) {
						if (cell instanceof ProMGraphCell) {
							selectedPatternNode = (PatternTreeNode) (((ProMGraphCell) cell).getNode());
							//String newLabel = selectedPatternNode.getLabel();
							String newLabel = selectedPatternNode.getNameToEdit();
							beforeModifiedPatterLabel = newLabel;
							orinalLabelOfCurrentPattern = newLabel;
							int patternUsageMode = setInformationOfSelectedPattern(newLabel);
							switch (patternUsageMode) {
								case NOT_USE_YET :
									notToUseRadioButton.setSelected(true);
									break;
								case SINGLE_PATTERN_MODE :
									desireNodeRadioButton.setSelected(true);
									break;
								case ABSTRACTION_MODE :
									desireSubGraphRadioButton.setSelected(true);
									break;
								default :
									notToUseRadioButton.setSelected(true);
									break;
							}
							//	editPanel.revalidate();
							//  editPanel.repaint();
						}
					}
				}
				editPanel.revalidate();
				editPanel.repaint();
			}
		});
	}

	//set the detail information of the pattern user selected
	public int setInformationOfSelectedPattern(String nameToEdit) {
		//String oldLabel = newOldPatternNameMap.get(nameToEdit).toString();
		Set<Set<String>> oldPatternNames = new HashSet<Set<String>>();
		Set<String> oldLabel = new HashSet<String>();
		if (newOldPatternNameMap.containsKey(nameToEdit)) {
			oldPatternNames = newOldPatternNameMap.get(nameToEdit);
			oldLabel = oldPatternNames.iterator().next();
		}
		/*
		 * else if (selectedPatternNameAlphabetMap.containsKey(nameToEdit)){
		 * oldPatternNames = selectedPatternNameAlphabetMap.get(nameToEdit);
		 * oldLabel = FuzzyMapUtil.findLongestLabel(oldPatternNames); }
		 */

		List<String> decodedActivityNames = ptnLabelDecodedActivityNameMap.get(oldLabel.toString());
		activityListModel.removeAllElements();
		//treat all the pattern in the same way
		nameField.setEnabled(true);
		nameField.setText(nameToEdit);
		for (int i = 0; i < decodedActivityNames.size(); i++) {
			activityListModel.addElement(decodedActivityNames.get(i));
		}
		//set the state of radio button group
		int patternUsageMode = judgeUsageModeOfPattern(oldLabel);
		return patternUsageMode;

		//   editPanel.revalidate();
		//editPanel.repaint();

	}

	private int judgeUsageModeOfPattern(Set<String> patternOldLabel) {
		int mode = -1;
		if (selectedPatternAlphabetNewLabel.containsKey(patternOldLabel)) {
			String patternName = selectedPatternAlphabetNewLabel.get(patternOldLabel);
			Set<Set<String>> allAlphabet = selectedPatternNameAlphabetMap.get(patternName);
			int numOfAllAlphbet = allAlphabet.size();
			if (numOfAllAlphbet > 1) {
				mode = ABSTRACTION_MODE;
			}
			if (numOfAllAlphbet == 1) {
				mode = SINGLE_PATTERN_MODE;
			}
		} else {
			mode = NOT_USE_YET;
		}
		return mode;
	}

	public void displayFuzzyMap(XLog transformedLog) {
		JPanel fuzzymodelPanel = (JPanel) createFuzzyMapPanel(transformedLog);
		parentPanel.removeAll();
		parentPanel.add(fuzzymodelPanel, BorderLayout.CENTER);
		parentPanel.validate();
		parentPanel.repaint();
	}

	public JComponent createFuzzyMapPanel(XLog transformedLog) {
		//get the parameters to mine the fuzzy map  

		int maxNumOfInitialPrimitiveNodes = 7;
		//prepare mining the fuzzy map 
		long time = System.currentTimeMillis();
		try {
			metrics.apply(transformedLog, attenuation, maxRelDistance.intValue(), context);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		time = System.currentTimeMillis() - time;
		String logStr = new String("Fuzzy Miner: Building repository took " + time + " ms.");
		context.log(logStr, MessageLevel.NORMAL);
		FuzzyBusinessProcessMap resultFMMap = new FuzzyBusinessProcessMap(metrics, selectedPatternNameAlphabetMap,
				selectedPatternNameSequenceMap, charActivityMap, log);
		FuzzyModelInformationPanel fuzzyModelPanel = new FuzzyModelInformationPanel(context, resultFMMap, context
				.getProgress(), transformedLog, metrics, maxNumOfInitialPrimitiveNodes);
		return fuzzyModelPanel;
	}

	public void setupEditPanel() {
		editPanel = new JPanel();
		editPanel.setBorder(BorderFactory.createEmptyBorder());
		editPanel.setPreferredSize(new Dimension(300, 450));
		editPanel.setMaximumSize(new Dimension(300, 450));
		editPanel.setMinimumSize(new Dimension(300, 450));
		editPanel.setBackground(background);
		//	editPanel.setOpaque(false);

		JLabel title = new JLabel("Edit Pattern");
		title.setFont(title.getFont().deriveFont(15f));
		title.setHorizontalAlignment(SwingConstants.CENTER);
		title.setHorizontalTextPosition(SwingConstants.CENTER);
		title.setAlignmentX(Component.CENTER_ALIGNMENT);
		title.setLocation(100, 30);
		editPanel.add(Box.createHorizontalStrut(10));
		editPanel.add(FuzzyGraphEditorPanel.centerHorizontally(title, 40));
		//editPanel.add(title);
		//pattern name
		nameField = new JTextField("");
		//nameField.getDocument().addDocumentListener(new NameDocumentListener());
		nameField.addKeyListener(new KeyAdapter() {
			public void keyTyped(KeyEvent e) {
				// live update of node property changes
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {

						//When there is no duplicate pattern and the  name of the current pattern has been changed
						//the current pattern will be deleted from the desired pattern list
						if (!isThereDuplicatePattern && patternListModel.contains(beforeModifiedPatterLabel)) {
							patternListModel.removeElement(beforeModifiedPatterLabel);
						}
						updateGraphNodeLabel();
						notToUseRadioButton.setSelected(true);
						beforeModifiedPatterLabel = nameField.getText();
					}
				});
			}
		});
		nameField.setFont(nameField.getFont().deriveFont(11f));
		nameField.setBackground(new Color(180, 180, 180));
		nameField.setForeground(new Color(10, 10, 10));
		nameField.setSize(80, 20);
		nameField.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
		nameLabel = new JLabel("Name:");
		nameLabel.setMaximumSize(new Dimension(55, 25));
		nameLabel.setPreferredSize(new Dimension(55, 25));
		//	nameLabel.setFont(nameLabel.getFont().deriveFont(11f));
		JPanel namePanel = FuzzyGraphEditorPanel.layoutHorizontally(new Component[] { nameLabel,
				Box.createHorizontalStrut(5), nameField }, 20);
		//namePanel.setLocation(15, 15);
		//activities in the pattern
		activityLabel = new JLabel("Activities:");
		activityLabel.setMaximumSize(new Dimension(55, 25));
		activityLabel.setPreferredSize(new Dimension(55, 25));
		activityLabel.setOpaque(false);
		//	activityLabel.setFont(nameLabel.getFont().deriveFont(11f));
		activityListModel = new DefaultListModel();
		activityList = new JList(activityListModel);
		//	activityList.setSelectionInterval(0, activityNameString.length - 1);
		JScrollPane activityListScrollPanel = new JScrollPane();
		activityListScrollPanel.setPreferredSize(new Dimension(100, 150));
		activityListScrollPanel.setViewportView(activityList);

		JPanel activityPanel = new JPanel();
		activityPanel.setBorder(BorderFactory.createEmptyBorder());
		activityPanel.setOpaque(false);
		activityPanel.setLayout(new BoxLayout(activityPanel, BoxLayout.X_AXIS));
		activityPanel.add(activityLabel);
		activityPanel.add(Box.createHorizontalStrut(10));
		activityPanel.add(activityListScrollPanel);
		activityPanel.add(Box.createHorizontalGlue());
		Point actPanelPosition = activityPanel.getLocation();

		JLabel usingModeLabel = new JLabel("Usage Mode");
		usingModeLabel.setOpaque(false);
		usingModeLabel.setLocation(actPanelPosition.x - 50, actPanelPosition.y + 200);
		notToUseRadioButton = new JRadioButton("Not to be used");
		notToUseRadioButton.setUI(new SlickerRadioButtonUI());
		//	notToUseRadioButton.setBorder(BorderFactory.createEmptyBorder(2, 10, 2, 2));
		notToUseRadioButton.setOpaque(false);
		notToUseRadioButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		notToUseRadioButton.setHorizontalAlignment(SwingConstants.LEFT);
		notToUseRadioButton.setSelected(true);
		notToUseRadioButton.setToolTipText("<html> Don't use the selected pattern to preprocess the log.</html>");
		notToUseRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Set<Set<String>> ptnNameAlphabet = new HashSet<Set<String>>();
				if (!isThereDuplicatePattern) {
					String patternName = nameField.getText();
					Set<String> selectedNodeName = selectedPatternNode.getLabelAlphabet();
					//	ptnNameAlphabet = newOldPatternNameMap.get(patternName);

					if (notToUseRadioButton.isSelected()) {
						selectedPatternNode.getAttributeMap().put(AttributeMap.FILLCOLOR, Color.WHITE);
						//if the selected node has been selected to transform the log later 
						if (selectedPatternNameAlphabetMap.containsKey(patternName)) {
							ptnNameAlphabet = selectedPatternNameAlphabetMap.get(patternName);
							if (ptnNameAlphabet.size() > 1) {
								//The selected node was used in the "Abstraction Mode" before
								//change the color of the nodes in the sub tree
								setColorOfNodesInSubTree(selectedPatternNode, false);
							}
							patternListModel.removeElement(nameField.getText());
							//remove the corresponding data because the pattern with this name will not be used
							selectedPatternNameAlphabetMap.remove(patternName);
							//oldLabelsOfSelectedPatterns.remove(selectedPatternNode.getLabelAlphabet());
							for (Set<String> offspringPtnName : ptnNameAlphabet) {
								oldLabelsOfSelectedPatterns.remove(offspringPtnName);
								selectedPatternAlphabetNewLabel.remove(offspringPtnName);
							}
							//newOldPatternNameMap.remove(patternName);
							parentOffspringNodesMap.remove(selectedPatternNode);
						} else {
							//the node was not be used a member of abstraction mode or
							//the node has not been used before
							if (selectedPatternAlphabetNewLabel.containsKey(selectedNodeName)) {
								String parentNodeName = selectedPatternAlphabetNewLabel.get(selectedNodeName);
								PatternTreeNode parentNode = findOriginalParentNode(selectedPatternNode);
								selectedPatternNameAlphabetMap.get(parentNodeName).remove(selectedNodeName);
								oldLabelsOfSelectedPatterns.remove(selectedNodeName);
								selectedPatternAlphabetNewLabel.remove(selectedNodeName);
								parentOffspringNodesMap.get(parentNode).remove(selectedPatternNode);
							}
						}

					}
					patternTree.signalViews();
				} else {
					JOptionPane.showMessageDialog(editPanel, "Please give another name to the current pattern!");
					notToUseRadioButton.setSelected(false);
				}
			}
		});

		desireNodeRadioButton = new JRadioButton("Single Pattern Mode");
		desireNodeRadioButton.setUI(new SlickerRadioButtonUI());
		//edgesBestRadioButton.setFont(this.smallFont);
		//desireNodeRadioButton.setBorder(BorderFactory.createEmptyBorder(2, 10, 2, 2));
		desireNodeRadioButton.setOpaque(false);
		//	desireNodeRadioButton.setForeground(COLOR_FG);
		desireNodeRadioButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		desireNodeRadioButton.setHorizontalAlignment(SwingConstants.LEFT);
		desireNodeRadioButton.setSelected(false);
		desireNodeRadioButton.setToolTipText("<html> Only use the selected pattern to preprocess the log.</html>");

		desireNodeRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Set<PatternTreeNode> offSpringNodes = new HashSet<PatternTreeNode>();
				Set<Set<String>> ptnNameAlphabet = new HashSet<Set<String>>();
				Set<String> selectedPtnAlphabet = selectedPatternNode.getLabelAlphabet();
				offSpringNodes.add(selectedPatternNode);
				if (!isThereDuplicatePattern) {
					String patternName = nameField.getText();
					if (desireNodeRadioButton.isSelected()) {
						//add the current pattern name into the patters list
						if (patternListModel.contains(patternName)) {
							setColorOfNodesInSubTree(selectedPatternNode, false);
						} else {
							patternListModel.addElement(patternName);
						}

						//this pattern was used in an abstraction mode before

						ptnNameAlphabet.add(selectedPtnAlphabet);
						if (oldLabelsOfSelectedPatterns.contains(selectedPtnAlphabet)) {
							String message = new String("<html> Are you sure to change the usage mode of the Pattern '"
									+ selectedPatternNode.getNameToEdit() + "'" + " as the Single Pattern mode?");
							//there exists conflict pattern nodes
							int chooseResult = JOptionPane.showConfirmDialog(wholePanel, message,
									"Conflict Patterns Confirm Dialog", JOptionPane.YES_NO_OPTION);
							if (chooseResult == JOptionPane.YES_OPTION) {
								setParentOffspringNodeMap(selectedPatternNode, offSpringNodes, ptnNameAlphabet,
										offSpringNodes, ptnNameAlphabet, patternName, true);
								selectedPatternNode.getAttributeMap().put(AttributeMap.FILLCOLOR, Color.green);
							} else {
								desireSubGraphRadioButton.setSelected(true);
							}
						} else {
							selectedPatternNameAlphabetMap.put(patternName, ptnNameAlphabet);
							selectedPatternAlphabetNewLabel.put(selectedPatternNode.getLabelAlphabet(), patternName);
							oldLabelsOfSelectedPatterns.add(selectedPatternNode.getLabelAlphabet());
							//newOldPatternNameMap.put(patternName, ptnNameAlphabet);						
							parentOffspringNodesMap.put(selectedPatternNode, offSpringNodes);
							selectedPatternNode.getAttributeMap().put(AttributeMap.FILLCOLOR, Color.green);
						}

					}
					patternTree.signalViews();
				} else {
					JOptionPane.showMessageDialog(editPanel, "Please give another name to the current pattern!");
					desireNodeRadioButton.setSelected(false);
				}
			}
		});

		//select all the offspring patterns of this pattern as one pattern,
		//i.e. use the patterns in the subtree whose root is this pattern to transform the log 
		desireSubGraphRadioButton = new JRadioButton("Abstraction Mode");
		desireSubGraphRadioButton.setUI(new SlickerRadioButtonUI());
		//edgesBestRadioButton.setFont(this.smallFont);
		//desireSubGraphRadioButton.setBorder(BorderFactory.createEmptyBorder(2, 10, 2, 2));
		desireSubGraphRadioButton.setOpaque(false);
		//	desireNodeRadioButton.setForeground(COLOR_FG);
		desireSubGraphRadioButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		desireSubGraphRadioButton.setHorizontalAlignment(SwingConstants.LEFT);
		desireSubGraphRadioButton.setSelected(false);
		desireSubGraphRadioButton.setToolTipText("<html> Use all the patterns in the subtree whose <br>"
				+ "root is the selected pattern to preprocess the log.</html>");

		desireSubGraphRadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (!isThereDuplicatePattern) {
					String patternName = nameField.getText();
					if (desireSubGraphRadioButton.isSelected()) {
						//add the current pattern name into the patters list
						if (!patternListModel.contains(patternName)) {
							patternListModel.addElement(patternName);
						}
						Set<PatternTreeNode> offSpringNodes = getOffspringNodes(selectedPatternNode);
						Set<Set<String>> offSpringLabelAlphabetSets = getOffspringNodesAlphbet(offSpringNodes);
						Set<Set<String>> confictPatternsAlphabets;
						Set<PatternTreeNode> conflictNodes = getConflictNodes(selectedPatternNode, offSpringNodes);
						confictPatternsAlphabets = getConflictNodesAlphabet(conflictNodes);
						if (confictPatternsAlphabets.size() == 0) {
							setParentOffspringNodeMap(selectedPatternNode, offSpringNodes, offSpringLabelAlphabetSets,
									conflictNodes, confictPatternsAlphabets, patternName, false);
						} else if (confictPatternsAlphabets.size() > 0) {
							/*
							 * Set<String> theConflictAlphabetLabel =
							 * FuzzyMapUtil
							 * .findLongestLabel(confictPatternsAlphabets);
							 * String theConflictAlphabet= new String("");
							 * if(selectedPatternAlphabetNewLabel
							 * .containsKey(theConflictAlphabetLabel)){
							 * theConflictAlphabet =
							 * selectedPatternAlphabetNewLabel
							 * .get(theConflictAlphabetLabel);
							 * System.out.println
							 * ("The pattern name of the alphabet " +
							 * theConflictAlphabetLabel + ": " +
							 * theConflictAlphabet); }
							 */
							String conflictNodesLabel = new String("");
							for (PatternTreeNode cfltNode : conflictNodes) {
								conflictNodesLabel += cfltNode.getNameToEdit() + ",";
							}
							conflictNodesLabel = conflictNodesLabel.substring(0, conflictNodesLabel.length());
							String message = new String("<html> Do you want to substitute the Pattern '"
									+ conflictNodesLabel + "'" + " to the Pattern" + "'" + patternName + "' ?");
							//there exists conflict pattern nodes
							int chooseResult = JOptionPane.showConfirmDialog(wholePanel, message,
									"Conflict Pattern Confirm Dialog", JOptionPane.YES_NO_OPTION);
							if (chooseResult == JOptionPane.YES_OPTION) {
								setParentOffspringNodeMap(selectedPatternNode, offSpringNodes,
										offSpringLabelAlphabetSets, conflictNodes, confictPatternsAlphabets,
										patternName, true);
							} else {
								setParentOffspringNodeMap(selectedPatternNode, offSpringNodes,
										offSpringLabelAlphabetSets, conflictNodes, confictPatternsAlphabets,
										patternName, false);
							}
						}

						selectedPatternNode.getAttributeMap().put(AttributeMap.FILLCOLOR, Color.GREEN);
						setColorOfNodesInSubTree(selectedPatternNode, true);

					}
					/*
					 * else { //remove the current pattern name into the patters
					 * list
					 * selectedPatternNode.getAttributeMap().put(AttributeMap
					 * .FILLCOLOR, Color.WHITE);
					 * setColorOfNodesInSubTree(selectedPatternNode,false);
					 * patternListModel.removeElement(nameField.getText());
					 * newOldPatternNameMap.remove(patternName);
					 * parentOffspringNodesMap.remove(selectedPatternNode); }
					 */
					patternTree.signalViews();
				} else {
					JOptionPane.showMessageDialog(editPanel, "Please give another name to the current pattern!");
					desireSubGraphRadioButton.setSelected(false);
				}
			}
		});

		ButtonGroup radioEdgesGroup = new ButtonGroup();
		radioEdgesGroup.add(notToUseRadioButton);
		radioEdgesGroup.add(desireNodeRadioButton);
		radioEdgesGroup.add(desireSubGraphRadioButton);

		JPanel choosePatternPanel = new JPanel();
		choosePatternPanel.setBorder(BorderFactory.createEmptyBorder());
		choosePatternPanel.setOpaque(false);
		//choosePatternPanel.setLayout()
		choosePatternPanel.setLayout(new BoxLayout(choosePatternPanel, BoxLayout.Y_AXIS));
		choosePatternPanel.add(usingModeLabel);
		choosePatternPanel.add(Box.createVerticalStrut(2));
		choosePatternPanel.add(notToUseRadioButton);
		choosePatternPanel.add(Box.createVerticalStrut(5));
		choosePatternPanel.add(desireNodeRadioButton);
		choosePatternPanel.add(Box.createVerticalStrut(5));
		choosePatternPanel.add(desireSubGraphRadioButton);
		//choosePatternPanel.setLocation(15, 300);

		//assemble the editor panel
		Box vbox = Box.createVerticalBox();
		vbox.add(Box.createRigidArea(new Dimension(5, 20)));
		vbox.add(title);
		vbox.add(Box.createVerticalStrut(20));
		vbox.add(namePanel);
		vbox.add(Box.createVerticalStrut(20));
		vbox.add(activityPanel);
		vbox.add(Box.createVerticalStrut(20));
		vbox.add(choosePatternPanel);
		vbox.add(Box.createVerticalStrut(20));
		//vbox.add(Box.createVerticalGlue());*/
		editPanel.add(vbox, BorderLayout.CENTER);
	}

	/*
	 * check whether each offspring node of the selected node has been used
	 * before
	 */
	private Set<Set<String>> getConflictNodesAlphabet(Set<PatternTreeNode> conflictNodes) {
		Set<Set<String>> confictPatternsAlphabet = new HashSet<Set<String>>();
		//Set<Set<String>> offSpringLabelAlphabetSets = getOffspringNodesAlphbet(selectedNode);
		for (PatternTreeNode node : conflictNodes) {
			confictPatternsAlphabet.add(node.getLabelAlphabet());
		}
		return confictPatternsAlphabet;
	}

	private Set<PatternTreeNode> getConflictNodes(PatternTreeNode selectedNode, Set<PatternTreeNode> offSpringNodes) {
		Set<PatternTreeNode> conflictNodes = new HashSet<PatternTreeNode>();
		//	Set<PatternTreeNode> offSpringNodes = getOffspringNodes(selectedNode);
		offSpringNodes.add(selectedNode);
		for (PatternTreeNode node : offSpringNodes) {
			Set<String> nodeAlphabet = node.getLabelAlphabet();
			if (oldLabelsOfSelectedPatterns.contains(nodeAlphabet)) {
				conflictNodes.add(node);
			}
		}
		return conflictNodes;
	}

	/*
	 * get the offspring nodes of the selected node
	 */
	private Set<PatternTreeNode> getOffspringNodes(PatternTreeNode selectedNode) {
		Set<PatternTreeNode> offSpringNodes = new HashSet<PatternTreeNode>();
		//find all the offspring pattern nodes
		Set<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> inEdges = ((PatternTreeImpl) patternTree)
				.getInEdges(selectedNode);

		Set<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> tmpInEdges = new HashSet<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>>();
		tmpInEdges.addAll(inEdges);
		while (!tmpInEdges.isEmpty()) {
			tmpInEdges.clear();
			for (PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode> inEdge : inEdges) {
				PatternTreeNode offspringNode = inEdge.getSource();
				if (offspringNode != null) {
					offSpringNodes.add(offspringNode);
					Set<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> offSprInEdges;
					offSprInEdges = ((PatternTreeImpl) patternTree).getInEdges(offspringNode);
					if (!offSprInEdges.isEmpty()) {
						tmpInEdges.addAll(offSprInEdges);
					}
				}
			}
			inEdges.clear();
			inEdges.addAll(tmpInEdges);
		}
		return offSpringNodes;
	}

	/*
	 * If the pattern is used in the abstraction mode, this method will get all
	 * the alphabet in the subtree.
	 */
	private Set<Set<String>> getOffspringNodesAlphbet(Set<PatternTreeNode> offSpringNodes) {
		Set<Set<String>> offSpringLabelAlphabetSets = new HashSet<Set<String>>();
		//Set<PatternTreeNode> offSpringNodes = getOffspringNodes(selectedNode);
		for (PatternTreeNode node : offSpringNodes) {
			offSpringLabelAlphabetSets.add(node.getLabelAlphabet());
		}
		return offSpringLabelAlphabetSets;
	}

	/*
	 * Set the map between the parent node and its offspring nodes
	 */
	private void setParentOffspringNodeMap(PatternTreeNode selectedNode, Set<PatternTreeNode> offSpringNodes,
			Set<Set<String>> offSpringLabelAlphabetSets, Set<PatternTreeNode> conflictNodes,
			Set<Set<String>> conflictPatternAlphabet, String newPatternName, boolean isSubstiuteOriginalPattern) {

		//get selected node's alphabet
		Set<String> selectedNodeLabelAlphabet = selectedNode.getLabelAlphabet();
		offSpringLabelAlphabetSets.add(selectedNodeLabelAlphabet);
		if (conflictPatternAlphabet.size() > 0) {
			if (isSubstiuteOriginalPattern) {

				//make the conflict pattern alphabet correspond to its new pattern name
				for (Set<String> conflictPattern : conflictPatternAlphabet) {
					String originalPatternName = selectedPatternAlphabetNewLabel.get(conflictPattern);
					selectedPatternNameAlphabetMap.get(originalPatternName).remove(conflictPattern);
					if (selectedPatternAlphabetNewLabel.containsKey(conflictPattern)) {
						selectedPatternAlphabetNewLabel.remove(conflictPattern);
						selectedPatternAlphabetNewLabel.put(conflictPattern, newPatternName);
					}
				}
				PatternTreeNode firstConflictNode = conflictNodes.iterator().next();
				if ((conflictPatternAlphabet.size() == 1) && parentOffspringNodesMap.containsKey(firstConflictNode)) {
					//if the usage mode of the selected node was changed from the abstraction node to single pattern node 
					//we should delete the whole map between the parent and its offsprings 
					Set<PatternTreeNode> offspringnodes = parentOffspringNodesMap.get(firstConflictNode);
					//	selectedPatternAlphabetNewLabel.remove(firstConflictNode.getLabelAlphabet());
					for (PatternTreeNode offspring : offspringnodes) {
						oldLabelsOfSelectedPatterns.remove(offspring.getLabelAlphabet());
						selectedPatternAlphabetNewLabel.remove(offspring.getLabelAlphabet());
					}
					selectedPatternNameAlphabetMap.remove(firstConflictNode.getLabelAlphabet());
					parentOffspringNodesMap.remove(firstConflictNode);
				} else {
					//make the conflict nodes not the offsprings of its original parent node anymore
					for (PatternTreeNode conflictNode : conflictNodes) {
						PatternTreeNode originalParentNode = findOriginalParentNode(conflictNode);
						parentOffspringNodesMap.get(originalParentNode).remove(conflictNode);
					}
				}
			} else {
				//remove all the conflictNodes from the offspring nodes set of the current selected pattern node
				offSpringNodes.removeAll(conflictNodes);
				offSpringLabelAlphabetSets.removeAll(conflictPatternAlphabet);
			}
		}

		parentOffspringNodesMap.put(selectedNode, offSpringNodes);
		//	oldLabelsOfSelectedPatterns.add(selectedNodeLabelAlphabet);
		oldLabelsOfSelectedPatterns.addAll(offSpringLabelAlphabetSets);
		selectedPatternNameAlphabetMap.put(newPatternName, offSpringLabelAlphabetSets);
		for (Set<String> selectedPatternAlphabet : offSpringLabelAlphabetSets) {
			selectedPatternAlphabetNewLabel.put(selectedPatternAlphabet, newPatternName);
		}
	}

	/*
	 * find the original parent node
	 */
	private PatternTreeNode findOriginalParentNode(PatternTreeNode offspringNode) {
		PatternTreeNode parentNode = null;
		Set<Entry<PatternTreeNode, Set<PatternTreeNode>>> entryset = parentOffspringNodesMap.entrySet();
		for (Entry<PatternTreeNode, Set<PatternTreeNode>> entry : entryset) {
			if (entry.getValue().contains(offspringNode)) {
				parentNode = entry.getKey();
				break;
			}
		}
		return parentNode;
	}

	/*	 *
	 * According the status whether this node is selected or not, set the color
	 * of its offsprings
	 */
	private void setColorOfNodesInSubTree(PatternTreeNode selectedNode, boolean isDesiredPattern) {
		//find all the offspring pattern nodes
		Set<PatternTreeNode> offspringNodes = parentOffspringNodesMap.get(selectedNode);
		Color backgroundColor;
		if (isDesiredPattern) {
			backgroundColor = Color.MAGENTA;
		} else {
			backgroundColor = Color.WHITE;
		}
		if (!offspringNodes.isEmpty()) {
			for (PatternTreeNode offspringNode : offspringNodes) {
				offspringNode.getAttributeMap().put(AttributeMap.FILLCOLOR, backgroundColor);
			}
		}
	}

	public void updateGraphNodeLabel() {

		//adjust the color of the selected node according to its usage mode
		if (desireNodeRadioButton.isSelected()) {
			selectedPatternNode.getAttributeMap().put(AttributeMap.FILLCOLOR, Color.WHITE);
		}
		if (desireSubGraphRadioButton.isSelected()) {
			selectedPatternNode.getAttributeMap().put(AttributeMap.FILLCOLOR, Color.WHITE);
			setColorOfNodesInSubTree(selectedPatternNode, false);
		}

		//adjust the pattern node label in the graph	
		String oldNodeLabel = selectedPatternNode.getLabel();
		String oldNameToEdit = selectedPatternNode.getNameToEdit();
		String afterModifiedlabel = nameField.getText();
		Set<Set<String>> oldPatternLabelSet;
		String patternLabelToRemove;
		if (afterModifiedlabel.length() > 0) {
			selectedPatternNode.setNameToEdit(afterModifiedlabel);
			String newLabel = oldNodeLabel.replace(oldNameToEdit, afterModifiedlabel);
			selectedPatternNode.setLabel(newLabel);
			selectedPatternNode.getAttributeMap().put(AttributeMap.LABEL, newLabel);
			patternTree.signalViews();

			//adjust the maps accordingly for transform the log later		
			if (!newOldPatternNameMap.containsKey(afterModifiedlabel)) {
				//if (!selectedPatternNameAlphabetMap.containsKey(afterModifiedlabel)) {
				if (isThereDuplicatePattern || (beforeModifiedPatterLabel.length() == 0)) {
					/*
					 * if there is duplicate patterns before this modification,
					 * or the user has delete all the characters in the pattern
					 * label it is necessary to get the pattern alphabet
					 * according to the original label of the pattern
					 */
					patternLabelToRemove = orinalLabelOfCurrentPattern;
					orinalLabelOfCurrentPattern = afterModifiedlabel;
				} else {
					patternLabelToRemove = beforeModifiedPatterLabel;
				}
				oldPatternLabelSet = newOldPatternNameMap.get(patternLabelToRemove);
				newOldPatternNameMap.remove(patternLabelToRemove);
				//change the current used patterns
				//	oldLabelsOfSelectedPatterns.remove(oldPatternLabelSet);
				if (selectedPatternNameAlphabetMap.containsKey(patternLabelToRemove)) {
					Set<Set<String>> beforeModifiedPatternAlphabetSet = selectedPatternNameAlphabetMap
							.get(patternLabelToRemove);
					oldLabelsOfSelectedPatterns.removeAll(beforeModifiedPatternAlphabetSet);
					selectedPatternNameAlphabetMap.remove(patternLabelToRemove);
					for (Set<String> selectedPatternAlphabet : beforeModifiedPatternAlphabetSet) {
						selectedPatternAlphabetNewLabel.remove(selectedPatternAlphabet);
					}
				}
				newOldPatternNameMap.put(afterModifiedlabel, oldPatternLabelSet);
				//	selectedPatternNameAlphabetMap.put(afterModifiedlabel, oldPatternLabelSet);
				isThereDuplicatePattern = false;
			} else {
				if (!beforeModifiedPatterLabel.equals(afterModifiedlabel)) {
					isThereDuplicatePattern = true;
					orinalLabelOfCurrentPattern = beforeModifiedPatterLabel;
					JOptionPane
							.showMessageDialog(this,
									"There is already a pattern with this name,please give another name to the current pattern!");
					nameField.setFocusable(true);
				}
			}
		}
	}

	public void setupDesiredPatternPanel() {
		desiredPatternPanel = new JPanel();
		desiredPatternPanel.setBorder(BorderFactory.createEmptyBorder());
		desiredPatternPanel.setPreferredSize(new Dimension(300, 350));
		desiredPatternPanel.setMaximumSize(new Dimension(300, 350));
		desiredPatternPanel.setMinimumSize(new Dimension(300, 350));
		desiredPatternPanel.setBackground(background);

		patternLabel = new JLabel("Patterns To Use");
		patternLabel.setFont(patternLabel.getFont().deriveFont(15f));
		patternLabel.setHorizontalAlignment(SwingConstants.CENTER);
		patternLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		patternLabel.setAlignmentX(JLabel.CENTER_ALIGNMENT);
		desiredPatternPanel.add(FuzzyGraphEditorPanel.centerHorizontally(patternLabel, 30));

		patternListModel = new DefaultListModel();
		patternList = new JList(patternListModel);
		patternList.setSelectionInterval(0, 0);
		patternList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		//according to the pattern the user selected to set the pattern value in the Edit Panel
		MouseListener mouseListener = new MouseAdapter() {
			public void mouseClicked(MouseEvent mouseEvent) {
				JList theList = (JList) mouseEvent.getSource();
				//if (mouseEvent.getClickCount() == 1) {
				int index = theList.locationToIndex(mouseEvent.getPoint());
				if (index >= 0) {
					Object o = theList.getModel().getElementAt(index);
					String selectedPatterLabel = o.toString();
					beforeModifiedPatterLabel = selectedPatterLabel;
					//	System.out.println("From the List the selected pattern is : " + o.toString());
					//nameField.setText(o.toString());
					int patternUsageMode = setInformationOfSelectedPattern(selectedPatterLabel);
					switch (patternUsageMode) {
						case NOT_USE_YET :
							notToUseRadioButton.setSelected(true);
							break;
						case SINGLE_PATTERN_MODE :
							desireNodeRadioButton.setSelected(true);
							break;
						case ABSTRACTION_MODE :
							desireSubGraphRadioButton.setSelected(true);
							break;
						default :
							notToUseRadioButton.setSelected(true);
							break;
					}
					findTheCorrespondingPatterNode(selectedPatterLabel);
					editPanel.revalidate();
					editPanel.repaint();
				}
			}
			//}
		};
		patternList.addMouseListener(mouseListener);
		JScrollPane patternListScrollPanel = new JScrollPane();
		patternListScrollPanel.setPreferredSize(new Dimension(100, 150));
		patternListScrollPanel.setViewportView(patternList);
		JPanel patternPanel = new JPanel();
		patternPanel.add(patternListScrollPanel);
		patternPanel.setOpaque(false);
		SlickerButton transformLogButton = new SlickerButton("Start Mining Using Patterns");
		transformLogButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				XLog transformedLog = preProcessLog();
				//get the event count after transforming the log
				XLogInfo logSummary = XLogInfoFactory.createLogInfo(transformedLog);
				int eventCount = logSummary.getEventClasses().size();
				//set the number of events of the transformed log to the metrics object
				metrics.setNumberOfLogEvents(eventCount);
				//set the size of each metrics
				setMetricsSize(eventCount);
				displayFuzzyMap(transformedLog);

				/*
				 * if (checkPatternValidation()){ displayFuzzyMap(); } else{
				 * JOptionPane.showInternalMessageDialog(desiredPatternPanel,
				 * "Each activity is only allowed to appear in one pattern."); }
				 */
			}
		});
		JPanel transformLogPanel = new JPanel();
		transformLogPanel.setOpaque(false);
		transformLogPanel.add(transformLogButton);
		//assemble the desiredPatternPanel
		Box vbox = Box.createVerticalBox();
		vbox.add(Box.createRigidArea(new Dimension(5, 20)));
		vbox.add(patternLabel);
		vbox.add(Box.createVerticalStrut(25));
		vbox.add(patternPanel);
		vbox.add(Box.createVerticalStrut(25));
		vbox.add(transformLogPanel);
		vbox.add(Box.createVerticalStrut(25));
		//vbox.add(Box.createVerticalGlue());
		desiredPatternPanel.add(vbox, BorderLayout.CENTER);

	}

	/*
	 * according to the user alternation, substitute the corresponding
	 * subsequences in each trace with the pattern
	 */
	/**
	 * @return
	 */
	private XLog preProcessLog() {
		//to change later
		//prepare relative data for pre-processing the log
		int numberOfSelectedPattern = patternListModel.size();
		//newLabelsOfSelectedPattern = new HashSet<String>();

		selectedPatternNameSequenceMap = new HashMap<String, Set<String>>();
		//Collections of all the sequences corresponding to each selected pattern
		Set<String> sequencesOfSelectedPatterns = new HashSet<String>();
		//Map the pattern sequence with the new pattern name which is used to substitute this sequence
		Map<String, String> sequenceNewPatternLabelMap = new HashMap<String, String>();
		//patAlphabetSeqMap				
		for (int i = 0; i < numberOfSelectedPattern; i++) {
			String newPatternLabel = (String) patternListModel.get(i);
			//	newLabelsOfSelectedPattern.add(newPatternLabel);
			Set<Set<String>> oldPatternLabelSet = selectedPatternNameAlphabetMap.get(newPatternLabel);
			//	oldLabelsOfSelectedPatterns.addAll(oldPatternLabelSet);
			Set<String> ptnSequences = new HashSet<String>();
			for (Set<String> oldPatternLabel : oldPatternLabelSet) {
				if (patAlphabetSeqMap.containsKey(oldPatternLabel)) {
					ptnSequences.addAll(patAlphabetSeqMap.get(oldPatternLabel));
				} else {//length-1 pattern
					ptnSequences.add(oldPatternLabel.toString().substring(1, 1 + encodingLength));
				}
			}
			//	selectedPatternNameAlphabetMap.put(newPatternLabel, oldPatternLabelSet);
			selectedPatternNameSequenceMap.put(newPatternLabel, ptnSequences);
			sequencesOfSelectedPatterns.addAll(ptnSequences);
			for (String seq : ptnSequences) {
				sequenceNewPatternLabelMap.put(seq, newPatternLabel);
			}
		}

		//find the start symbol of the corresponding sub sequeces of each desired pattern
		EquivalenceClass equivalenceClass = new EquivalenceClass();
		Map<String, Set<String>> startSymbolSequenceOfPatternSetMap = equivalenceClass
				.getStartSymbolEquivalenceClassMap(encodingLength, sequencesOfSelectedPatterns);
		Set<String> startSymbolSequenceSet = new TreeSet<String>();

		//create a new log according to the selected patterns
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		XLog outputLog = factory.createLog();
		encodedTraceList = ptnAbsFrm.getEncodedTraceList();

		for (XExtension extension : log.getExtensions()) {
			outputLog.getExtensions().add(extension);
		}

		outputLog.setAttributes(log.getAttributes());
		XTrace originalTrace;
		XTrace outputTrace;
		XEvent outputEvent;
		String encodedTrace, currentSymbol;
		String outputEncodedTrace = new String("");
		int encodedTraceLength, noMatches, repeatLength;
		int noTraces = encodedTraceList.size();
		Map<Integer, XEvent> patternLocationEventMap = new HashMap<Integer, XEvent>();
		Pattern pattern;
		Matcher matcher;

		//test

		for (int i = 0; i < noTraces; i++) {
			originalTrace = log.get(i);
			outputTrace = factory.createTrace();
			outputTrace.setAttributes(originalTrace.getAttributes());

			for (XExtension extension : originalTrace.getExtensions()) {
				outputTrace.getExtensions().add(extension);
			}

			encodedTrace = encodedTraceList.get(i);
			//	encodedTrace = "rqfwfqwcujgndbvxk";
			encodedTraceLength = encodedTrace.length() / encodingLength;
			//encodedTraceLength = encodedTrace.length(); 
			if (DEBUG) {
				System.out.println("@@@@The trace to transfer is " + encodedTrace);
				System.out.println("@@@@The original length of this trace  is " + encodedTrace.length());
				System.out.println("@@@@The length of the current encodedTrace is " + encodedTraceLength);
			}
			//comment temporarily
			/*
			 * if (encodedTraceLength != originalTrace.size()) {
			 * ErrorDialog.showErrorDialog(this,
			 * "Number of Events Mismatch between originalTrace and encodedTrace"
			 * ); return null; }
			 */
			boolean isPatternSequenceFound = false;
			//			System.out.println("Trace "+(i+1)+": "+encodedTrace);
			for (int j = 0; j < encodedTraceLength; j++) {
				Integer patternIdx = new Integer(j);
				if (DEBUG) {
					System.out.println("@@@@The no of char being dealed with is " + j);
				}
				currentSymbol = encodedTrace.substring(j * encodingLength, (j + 1) * encodingLength);
				if (startSymbolSequenceOfPatternSetMap.containsKey(currentSymbol)) {
					startSymbolSequenceSet = startSymbolSequenceOfPatternSetMap.get(currentSymbol);
					//use the longer pattern first
					/*
					 * Vector<String> arrayOfSequenceWithSameStart =
					 * decideOrderToUseSequence(startSymbolSequenceSet, false);
					 * int lengthOfSeqArray =
					 * arrayOfSequenceWithSameStart.size(); // for(String repeat
					 * : startSymbolSequenceSet){ for (int k = 0; k <
					 * lengthOfSeqArray; k++) {
					 */
					for (String repeat : startSymbolSequenceSet) {
						////use the longer pattern first
						//String repeat = arrayOfSequenceWithSameStart.get(k);
						pattern = Pattern.compile("(" + repeat + "){1,}");
						matcher = pattern.matcher(encodedTrace);
						//if (matcher.find(j * encodingLength) && matcher.start() == j * encodingLength) {
						if (matcher.find(j * encodingLength) && (matcher.start() == j * encodingLength)) {
							//find the continuous repeat
							isPatternSequenceFound = true;
							repeatLength = repeat.length() / encodingLength;
							noMatches = (matcher.end() - matcher.start()) / repeat.length();
							//System.out.println("Repeat Found: "+repeat+" , "+noMatches);							
							//	int replaceIdx = noMatches * repeatLength/encodingLength - 1;
							j += noMatches * repeatLength - 1;
							String currentPatternName = sequenceNewPatternLabelMap.get(repeat);
							//	System.out.println("Repeat Found: " + repeat + " , " + noMatches + " , "
							//		+ currentPatternName);
							//outputEvent = (XEvent) originalTrace.get(j).clone();
							outputEvent = (XEvent) originalTrace.get(j).clone();
							if (XConceptExtension.instance() == null) {
								ErrorDialog.showErrorDialog(this, "Null Concept Extension");
							}
							XAttribute name = factory.createAttributeLiteral("concept:name", currentPatternName,
									XConceptExtension.instance());
							outputEvent.getAttributes().put("concept:name", name);
							XAttribute type = factory.createAttributeLiteral("lifecycle:transition", "complete",
									XConceptExtension.instance());
							outputEvent.getAttributes().put("lifecycle:transition", type);
							patternLocationEventMap.put(patternIdx, outputEvent);
							outputEncodedTrace += currentPatternName;
							outputTrace.add(outputEvent);
							//previousAbstractionName = abstractionName;
						}
					}
					if (!isPatternSequenceFound) {
						/*
						 * if there is no pattern sequence starting with the
						 * current symbol being found, try to find whether there
						 * is non-continuous pattern. if there is not
						 * non-continuous pattern,then add the current symbol to
						 * the out trace
						 */
						//outputTrace.add(originalTrace.get(j));
						//r keeps the current index of the encoded trace
						//try to find non-continuous pattern
						/*
						 * for (int k = 0; k < lengthOfSeqArray; k++) { String
						 * currentPtnSeq = arrayOfSequenceWithSameStart.get(k);
						 */
						int rIdx = patternIdx;
						for (String currentPtnSeq : startSymbolSequenceSet) {
							//total length of current pattern Sequence. e.g. the length  of "a0b0" is 4
							int curPtnSeqLength = currentPtnSeq.length();
							Set<Integer> deletePtnCharPosition = new HashSet<Integer>();
							String curSubStringInRepeat, curSubStringInTrace;
							for (int m = 1 * encodingLength; m < curPtnSeqLength; m += encodingLength) {
								curSubStringInRepeat = currentPtnSeq.substring(m, m + encodingLength);
								for (int r = rIdx + 1 * encodingLength; r < encodedTraceLength; r += encodingLength) {
									curSubStringInTrace = encodedTrace.substring(r, r + encodingLength);
									if (curSubStringInRepeat.equals(curSubStringInTrace)) {
										deletePtnCharPosition.add(r);
										//m++;
										if (m == curPtnSeqLength - encodingLength) {
											//found a non-continuous pattern
											isPatternSequenceFound = true;
											//create new pattern event
											String currentPatternName = sequenceNewPatternLabelMap.get(currentPtnSeq);
											outputEvent = (XEvent) originalTrace.get(j).clone();
											if (XConceptExtension.instance() == null) {
												ErrorDialog.showErrorDialog(this, "Null Concept Extension");
											}
											XAttribute name = factory.createAttributeLiteral("concept:name",
													currentPatternName, XConceptExtension.instance());
											outputEvent.getAttributes().put("concept:name", name);
											XAttribute type = factory.createAttributeLiteral("lifecycle:transition",
													"complete", XConceptExtension.instance());
											outputEvent.getAttributes().put("lifecycle:transition", type);
											patternLocationEventMap.put(patternIdx, outputEvent);
											outputEncodedTrace += currentPatternName;
											outputTrace.add(outputEvent);
											//change the original encode trace by deleting the chars in the non-continuous pattern 
											String tmpSeq = new String("");
											XTrace tmpTrace = factory.createTrace();
											tmpTrace.setAttributes(originalTrace.getAttributes());

											for (XExtension extension : originalTrace.getExtensions()) {
												tmpTrace.getExtensions().add(extension);
											}

											//XTrace tmpTrace=originalTrace;
											for (int p = 0; p < encodedTraceLength; p++) {
												if (!deletePtnCharPosition.contains(p)) {
													String charString = encodedTrace.substring(p * encodingLength,
															(p + 1) * encodingLength);
													tmpSeq += charString;
													tmpTrace.add(originalTrace.get(p));
												}
											}
											encodedTrace = tmpSeq;
											encodedTraceLength = encodedTrace.length() / encodingLength;
											originalTrace = tmpTrace;
											System.out.println("After finding the non-continuous pattern "
													+ currentPtnSeq + ", the encoded trace had became " + encodedTrace);
											break;
										} else {
											rIdx += encodingLength;
											break;
										}
									}
								}
							}
						}
					}
					//there is neither continuous pattern sequence nor non-continuous pattern sequence
					if (!isPatternSequenceFound) {
						outputTrace.add(originalTrace.get(j));
						outputEncodedTrace += currentSymbol;
					}
					//reset the boolean value to false for next symbol
					isPatternSequenceFound = false;
				} else {
					outputEncodedTrace += currentSymbol;
					outputTrace.add(originalTrace.get(j));
				}
			}
			outputLog.add(outputTrace);
		}

		String transformLogFileName = "test.xes.gz";
		JFileChooser fileChooser = new JFileChooser(".");
		int returnValue = fileChooser.showSaveDialog(this);
		if (returnValue == JFileChooser.APPROVE_OPTION) {
			File file = fileChooser.getSelectedFile();
			if (file.exists()) {
				int response = JOptionPane.showConfirmDialog(this, "Overwrite existing file?", "Confirm overwrite",
						JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
				if (response == JOptionPane.CANCEL_OPTION) {
					return outputLog;
				}
			}
			transformLogFileName = file.getAbsolutePath();
		}

		try {

			FileOutputStream out = new FileOutputStream(transformLogFileName);

			XesXmlGZIPSerializer xesGzipSerializer = new XesXmlGZIPSerializer();
			xesGzipSerializer.serialize(outputLog, out);
			out.close();
		} catch (FileNotFoundException e) {
			ErrorDialog.showErrorDialog(this, e.getMessage());
		} catch (IOException e) {
			ErrorDialog.showErrorDialog(this, e.getMessage());
		}

		return outputLog;

	}

	/*
	 * decide the order of using the repeat sequences with the same start
	 * symbol.
	 * 
	 * @useShorterPatterFirst = true; The order to use the pattern sequence is
	 * from the shorter one to the longer one basically. Note that the sequence
	 * of length 1 will be used finally.
	 * 
	 * @useShorterPatterFirst = false; The order to use the pattern is from the
	 * longer one to the shorter one.
	 */
	private Vector<String> decideOrderToUseSequence(Set<String> startSymbolSequenceSet, boolean useShorterPatterFirst) {
		Object[] orderedSequence = startSymbolSequenceSet.toArray();
		Vector<String> tempSequeceArray = new Vector<String>();
		int totalLength = orderedSequence.length;
		tempSequeceArray.setSize(totalLength);
		if (useShorterPatterFirst) {
			int endOfLoop;
			String shortestSeq = orderedSequence[totalLength - 1].toString();
			if (shortestSeq.length() == 1) {
				//make the repeat sequence with length 1 be used finally
				tempSequeceArray.add(totalLength - 1, shortestSeq);
				//tempSequeceArray[totalLength - 1] = shortestSeq;
				endOfLoop = totalLength - 1;
			} else {
				endOfLoop = totalLength;
			}
			//change the order of the repeat sequences
			for (int i = 0; i < endOfLoop; i++) {
				tempSequeceArray.add(i, orderedSequence[endOfLoop - i - 1].toString());
			}
		} else {
			//int j = 0;
			for (String sequence : startSymbolSequenceSet) {
				tempSequeceArray.add(sequence);
				////j++;
			}

		}
		//test
		/*
		 * for (int j = 0; j < orderedSequence.length; j++) {
		 * System.out.println("ORDERED SEQUENCE No " + j + ": is " +
		 * tempSequeceArray.get(j)); }
		 */
		return tempSequeceArray;

	}

	private void setMetricsSize(int eventCount) {
		//set the size of the unary metrics
		for (UnaryMetric um : metrics.getUnaryMetrics()) {
			um.setSize(eventCount);
		}

		//set the size of the binary significance metrics
		for (BinaryLogMetric blm : metrics.getBinaryLogMetrics()) {
			blm.setSize(eventCount);
		}
		for (BinaryDerivateMetric bdm : metrics.getBinaryDerivateMetrics()) {
			bdm.setSize(eventCount);
		}

	}

	private void findTheCorrespondingPatterNode(String nodeLabel) {
		for (PatternTreeNode node : patternTree.getNodes()) {
			if (node.getNameToEdit().equals(nodeLabel)) {
				selectedPatternNode = node;
				selectedPatternNode.getAttributeMap().put(AttributeMap.EDGECOLOR, Color.red);
				patternTree.signalViews();
				//graphPanel.getGraph().set
				break;
			}
		}
	}
}
