package org.processmining.plugins.fuzzymap.ui;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.models.graphbased.directed.analysis.ShortestPathFactory;
import org.processmining.models.graphbased.directed.analysis.ShortestPathInfo;
import org.processmining.models.graphbased.directed.fuzzymodel.attenuation.Attenuation;
import org.processmining.models.graphbased.directed.fuzzymodel.attenuation.LinearAttenuation;
import org.processmining.models.graphbased.directed.fuzzymodel.attenuation.NRootAttenuation;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.Metric;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.MetricsRepository;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.binary.BinaryMetric;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.unary.UnaryMetric;
import org.processmining.models.graphbased.directed.patterntree.PatternTree;
import org.processmining.models.graphbased.directed.patterntree.PatternTreeEdge;
import org.processmining.models.graphbased.directed.patterntree.PatternTreeNode;
import org.processmining.plugins.abstractions.PatternAbstractionFrame;
import org.processmining.plugins.fuzzymap.FuzzyMapConnection;
import org.processmining.plugins.fuzzymap.util.FuzzyMapUtil;
import org.processmining.plugins.fuzzymodel.miner.ui.AttenuationDisplayPanel;
import org.processmining.plugins.fuzzymodel.miner.ui.MetricConfigurationComponent;
import org.processmining.plugins.ptnBasedPreprocess.PatternTreeImpl;
import org.processmining.plugins.ptnBasedPreprocess.util.PatternAbstractionFrameUtil;

import com.fluxicon.slickerbox.colors.SlickerColors;
import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.fluxicon.slickerbox.ui.SlickerRadioButtonUI;
import com.fluxicon.slickerbox.ui.SlickerSliderUI;

public class FuzzyMapMinerConfigurationUI {
	//	private UIPluginContext context;

	protected static final String METRICS_ALL = "show all metrics";
	protected static final String METRICS_TRACE = "show trace metrics";
	protected static final String METRICS_UNARY = "show unary significance metrics";
	protected static final String METRICS_BINSIG = "show binary significance metrics";
	protected static final String METRICS_BINCOR = "show binary correlation metrics";

	/*
	 * protected static Color COLOR_OUTER_BG = new Color(130, 130, 130);
	 * protected static Color COLOR_BG = new Color(120, 120, 120);
	 */

	protected MetricsRepository metrics;
	protected Attenuation attenuation;

	protected JComboBox metricsFilterBox;
	protected JScrollPane metricsScrollPane;
	protected JPanel metricsList;

	protected JRadioButton attLinearRadioButton;
	protected JRadioButton attNRootRadioButton;
	protected JSlider maxRelDistSlider;
	protected JSlider attNRootNSlider;
	protected AttenuationDisplayPanel attDisplayPanel;

	protected JCheckBox patternFrequencyCheckBox;
	protected JCheckBox instancePercentageFilterCheckBox;
	protected JCheckBox conservednessFilterCheckBox;
	protected JCheckBox alphabetSizeFilterCheckBox;
	protected JSlider patternFrequencySlider;
	protected JSlider instancePercentageFilterSlider;
	protected JSlider conservednessFilterSlider;
	protected JSlider alphabetSizeFilterSlider;
	protected JPanel patternFilterPanel;

	protected int minFrequencyThreshold = 0;
	protected int minInstanceCountPercentageThreshold = 0;
	protected int minAlphabetSize = 0;
	protected int minConservednessThreshold = 0;
	protected PatternAbstractionFrameUtil ptnAbsFrm;
	protected int encodingLength = 1;
	protected int[] initalFilterPatternParameters;
	protected UIPluginContext context;

	protected XLog log;

	protected Map<Integer, Set<PatternTreeNode>> sizeAlphabetSetMap;
	protected int[] sortedSizeArray;

	public FuzzyMapMinerConfigurationUI(final UIPluginContext context, XLog log) {
		this.context = context;
		this.log = log;
	}

	public JPanel getOptionsPanel() {
		// setup
		JPanel wholePanel = new JPanel();
		wholePanel.removeAll();
		wholePanel.setLayout(new BorderLayout());
		//	metrics = MetricsRepository.createRepository(summary);
		metrics = MetricsRepository.createRepository();
		// GUI setup
		metricsFilterBox = SlickerFactory.instance().createComboBox(
				new String[] { FuzzyMapMinerConfigurationUI.METRICS_ALL, FuzzyMapMinerConfigurationUI.METRICS_TRACE,
						FuzzyMapMinerConfigurationUI.METRICS_UNARY, FuzzyMapMinerConfigurationUI.METRICS_BINSIG,
						FuzzyMapMinerConfigurationUI.METRICS_BINCOR });
		metricsFilterBox.setOpaque(false);
		metricsFilterBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// adjust metric list's contents to combo box filter selection value
				metricsList = new JPanel();
				metricsList.setBackground(new Color(40, 40, 40));
				metricsList.setBorder(BorderFactory.createEmptyBorder());
				metricsList.setLayout(new BoxLayout(metricsList, BoxLayout.Y_AXIS));
				ArrayList<Metric> upData = new ArrayList<Metric>();
				String filter = (String) metricsFilterBox.getSelectedItem();
				if (filter.equals(FuzzyMapMinerConfigurationUI.METRICS_ALL)) {
					upData.addAll(metrics.getTraceMetrics());
					upData.addAll(metrics.getUnaryMetrics());
					upData.addAll(metrics.getSignificanceBinaryMetrics());
					upData.addAll(metrics.getCorrelationBinaryMetrics());
				} else if (filter.equals(FuzzyMapMinerConfigurationUI.METRICS_TRACE)) {
					upData.addAll(metrics.getTraceMetrics());
				} else if (filter.equals(FuzzyMapMinerConfigurationUI.METRICS_UNARY)) {
					upData.addAll(metrics.getUnaryMetrics());
				} else if (filter.equals(FuzzyMapMinerConfigurationUI.METRICS_BINSIG)) {
					upData.addAll(metrics.getSignificanceBinaryMetrics());
				} else if (filter.equals(FuzzyMapMinerConfigurationUI.METRICS_BINCOR)) {
					upData.addAll(metrics.getCorrelationBinaryMetrics());
				}
				for (Metric metric : upData) {
					metricsList.add(new MetricConfigurationComponent(metric));
				}
				metricsScrollPane.getViewport().setView(metricsList);
			}
		});
		metricsList = new JPanel();
		metricsList.setBorder(BorderFactory.createEmptyBorder());
		metricsList.setBackground(new Color(40, 40, 40));
		metricsList.setLayout(new BoxLayout(metricsList, BoxLayout.Y_AXIS));
		ArrayList<Metric> upData = new ArrayList<Metric>();
		upData.addAll(metrics.getTraceMetrics());
		upData.addAll(metrics.getUnaryMetrics());
		upData.addAll(metrics.getSignificanceBinaryMetrics());
		upData.addAll(metrics.getCorrelationBinaryMetrics());
		for (Metric metric : upData) {
			metricsList.add(new MetricConfigurationComponent(metric));
		}
		metricsScrollPane = new JScrollPane(metricsList);
		metricsScrollPane.setBorder(BorderFactory.createEmptyBorder());
		metricsScrollPane.setOpaque(false);
		metricsScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		metricsScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		//the metrics Panel for tab Panel
		RoundedPanel metricsPanel = new RoundedPanel(10, 5, 0);
		//metricsPanel.setBackground(new Color(80, 80, 80));
		metricsPanel.setBackground(SlickerColors.COLOR_BG_2);
		metricsPanel.setLayout(new BorderLayout());
		metricsPanel.setMinimumSize(new Dimension(480, 200));
		metricsPanel.setMaximumSize(new Dimension(1000, 2000));
		metricsPanel.setPreferredSize(new Dimension(480, 300));
		JPanel filterPanel = new JPanel();
		filterPanel.setOpaque(false);
		filterPanel.setLayout(new BorderLayout());
		filterPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 5, 10));
		JLabel filterLabel = new JLabel("Customize view: ");
		//filterLabel.setForeground(new Color(160, 160, 160));
		filterLabel.setForeground(SlickerColors.COLOR_FG);
		filterLabel.setOpaque(false);
		filterPanel.add(filterLabel, BorderLayout.WEST);
		filterPanel.add(metricsFilterBox, BorderLayout.CENTER);
		metricsPanel.add(filterPanel, BorderLayout.NORTH);
		metricsPanel.add(metricsScrollPane, BorderLayout.CENTER);
		// right panel
		ChangeListener attUpdateChangeListener = new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				updateAttenuationPanel();
			}
		};
		attenuation = new NRootAttenuation(2.7, 5);
		attLinearRadioButton = new JRadioButton("Linear attenuation", false);
		attLinearRadioButton.setUI(new SlickerRadioButtonUI());
		attLinearRadioButton.setOpaque(false);
		attLinearRadioButton.addChangeListener(attUpdateChangeListener);
		attNRootRadioButton = new JRadioButton("Nth root with radical", true);
		attNRootRadioButton.setUI(new SlickerRadioButtonUI());
		attNRootRadioButton.setOpaque(false);
		attNRootRadioButton.addChangeListener(attUpdateChangeListener);
		ButtonGroup attButtonGroup = new ButtonGroup();
		attButtonGroup.add(attLinearRadioButton);
		attButtonGroup.add(attNRootRadioButton);
		attNRootNSlider = new JSlider(SwingConstants.HORIZONTAL, 1000, 4000, 2700);
		//attNRootNSlider = new JSlider(JSlider.HORIZONTAL, 100, 400, 270);
		attNRootNSlider.setUI(new SlickerSliderUI(attNRootNSlider));
		attNRootNSlider.addChangeListener(attUpdateChangeListener);
		maxRelDistSlider = new JSlider(SwingConstants.HORIZONTAL, 1, 20, 4);
		maxRelDistSlider.setUI(new SlickerSliderUI(maxRelDistSlider));
		maxRelDistSlider.addChangeListener(attUpdateChangeListener);
		//attenuationDisplayPanel
		attDisplayPanel = new AttenuationDisplayPanel(attenuation, 5);
		attDisplayPanel.setBackground(Color.BLACK);
		RoundedPanel attDisplayRoundedPanel = RoundedPanel.enclose(attDisplayPanel, 10, 5, 0);
		attDisplayRoundedPanel.setMinimumSize(new Dimension(200, 150));
		attDisplayRoundedPanel.setPreferredSize(new Dimension(300, 200));
		attDisplayRoundedPanel.setMaximumSize(new Dimension(400, 300));
		JPanel confPanel = new JPanel();
		confPanel.setOpaque(false);
		confPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
		confPanel.setLayout(new BoxLayout(confPanel, BoxLayout.Y_AXIS));
		confPanel.setMaximumSize(new Dimension(800, 500));
		confPanel.setMinimumSize(new Dimension(230, 180));
		confPanel.setPreferredSize(new Dimension(250, 180));
		//Maximal event distance Panel
		JPanel maxDistPanel = new JPanel();
		maxDistPanel.setMaximumSize(new Dimension(1000, 40));
		//maxDistPanel.setMaximumSize(new Dimension(250, 40));
		maxDistPanel.setOpaque(false);
		maxDistPanel.setLayout(new BoxLayout(maxDistPanel, BoxLayout.X_AXIS));
		JLabel maxDistLabel = new JLabel("Maximal event distance:");
		maxDistLabel.setOpaque(false);
		maxDistPanel.add(maxDistLabel);
		maxDistPanel.add(Box.createHorizontalStrut(5));
		maxDistPanel.add(maxRelDistSlider);
		//Select attenuation Header Panel
		JPanel attHeaderPanel = new JPanel();
		attHeaderPanel.setMaximumSize(new Dimension(1000, 30));
		//attHeaderPanel.setMaximumSize(new Dimension(250, 30));
		attHeaderPanel.setOpaque(false);
		attHeaderPanel.setLayout(new BoxLayout(attHeaderPanel, BoxLayout.X_AXIS));
		JLabel attHeaderLabel = new JLabel("Select attenuation to use:");
		attHeaderLabel.setOpaque(false);
		attHeaderPanel.add(attHeaderLabel);
		attHeaderPanel.add(Box.createHorizontalGlue());
		//NRoot Panel
		JPanel attNRootPanel = new JPanel();
		attNRootPanel.setMaximumSize(new Dimension(1000, 40));
		attNRootPanel.setOpaque(false);
		attNRootPanel.setLayout(new BoxLayout(attNRootPanel, BoxLayout.X_AXIS));
		//attNRootPanel.setBorder(BorderFactory.createEmptyBorder(30, 15, 15, 15));
		attNRootPanel.add(Box.createHorizontalStrut(30));
		attNRootPanel.add(attNRootRadioButton);
		attNRootPanel.add(Box.createHorizontalStrut(5));
		attNRootPanel.add(attNRootNSlider);
		//Linear attenuation Panel
		JPanel attLinearPanel = new JPanel();
		attLinearPanel.setMaximumSize(new Dimension(1000, 40));
		//attLinearPanel.setMaximumSize(new Dimension(250, 40));
		attLinearPanel.setOpaque(false);
		attLinearPanel.setLayout(new BoxLayout(attLinearPanel, BoxLayout.X_AXIS));
		attLinearPanel.add(Box.createHorizontalStrut(30));
		attLinearPanel.add(attLinearRadioButton);
		attLinearPanel.add(Box.createHorizontalGlue());
		// assemble conf. panel
		confPanel.add(maxDistPanel);
		confPanel.add(Box.createVerticalStrut(20));
		confPanel.add(attHeaderPanel);
		confPanel.add(Box.createVerticalStrut(15));
		confPanel.add(attNRootPanel);
		confPanel.add(Box.createVerticalStrut(10));
		confPanel.add(attLinearPanel);
		// assemble right hand panel
		//SmoothPanel rightSplit = new SmoothPanel();
		JPanel rightSplit = new JPanel();
		rightSplit.setOpaque(false);
		rightSplit.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
		/*
		 * rightSplit.setBackground(COLOR_BG); rightSplit.setLayout(new
		 * BorderLayout()); rightSplit.add(attDisplayRoundedPanel,
		 * BorderLayout.WEST); rightSplit.add(confPanel, BorderLayout.CENTER);
		 */

		//ljf change the layout method of the measurement tab panel
		double size1[][] = { { 200, 300 }, { TableLayoutConstants.FILL } };
		rightSplit.setLayout(new TableLayout(size1));
		rightSplit.add(attDisplayRoundedPanel, "0,0");
		rightSplit.add(confPanel, "1,0");

		//create the frequency parameter
		ptnAbsFrm = new PatternAbstractionFrameUtil(context, log);
		ptnAbsFrm.findRepeatPatternsInLog();
		encodingLength = ptnAbsFrm.getEncodingLength();
		initalFilterPatternParameters = ptnAbsFrm.prepareInitialFilterParameters();

		//create the pattern filter panel
		patternFrequencyCheckBox = new JCheckBox("Frequency", true);
		patternFrequencyCheckBox.setOpaque(false);
		patternFrequencyCheckBox.setSelected(true);

		patternFrequencySlider = new JSlider(SwingConstants.HORIZONTAL, 0, 100, 0);
		patternFrequencySlider.setUI(new SlickerSliderUI(patternFrequencySlider));
		patternFrequencySlider.setEnabled(true);
		Dimension d = patternFrequencySlider.getPreferredSize();
		patternFrequencySlider.setPreferredSize(new Dimension(d.width, d.height));
		//Set the initial value of the patternFrequencySlider to the average frequency
		patternFrequencySlider.setValue(initalFilterPatternParameters[0]);
		patternFrequencySlider
				.setToolTipText("<html>Filters the pattern alphabets based on their absolute non-overlapping counts.<br>"
						+ "Pattern alphabets below the threshold will be ignored.</html>");
		minFrequencyThreshold = initalFilterPatternParameters[0];

		final JLabel frequencyFilterValueLabel = new JLabel("" + initalFilterPatternParameters[0] + "");
		frequencyFilterValueLabel.setEnabled(true);

		patternFrequencyCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (patternFrequencyCheckBox.isSelected()) {
					patternFrequencySlider.setEnabled(true);
					frequencyFilterValueLabel.setEnabled(true);
					minFrequencyThreshold = patternFrequencySlider.getValue();
				} else {
					patternFrequencySlider.setEnabled(false);
					frequencyFilterValueLabel.setEnabled(false);
					minFrequencyThreshold = 0;
				}
			}
		});

		patternFrequencySlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				frequencyFilterValueLabel.setText(patternFrequencySlider.getValue() + "");
				if (!patternFrequencySlider.getValueIsAdjusting()) {
					minFrequencyThreshold = patternFrequencySlider.getValue();
				}
			}
		});

		// Pattern Instance Count Parameter

		instancePercentageFilterCheckBox = new JCheckBox("Instance %");
		instancePercentageFilterCheckBox.setOpaque(false);
		instancePercentageFilterCheckBox.setSelected(true);

		instancePercentageFilterSlider = new JSlider(SwingConstants.HORIZONTAL, 0, 100, 0);
		instancePercentageFilterSlider.setUI(new SlickerSliderUI(instancePercentageFilterSlider));
		instancePercentageFilterSlider.setEnabled(true);
		d = instancePercentageFilterSlider.getPreferredSize();
		instancePercentageFilterSlider.setPreferredSize(new Dimension(d.width, d.height));
		//Set the initial value of the instancePercentageFilterSlider to the average instanceCount
		instancePercentageFilterSlider.setValue(initalFilterPatternParameters[1]);
		instancePercentageFilterSlider
				.setToolTipText("<html>Filter the pattern alphabets based on the percentage of instances in which they appear.<br>"
						+ " Pattern alphabets below the threshold will be ignored.</html>");
		minInstanceCountPercentageThreshold = initalFilterPatternParameters[1];
		final JLabel instancePercentageFilterValueLabel = new JLabel("" + initalFilterPatternParameters[1] + "");
		instancePercentageFilterValueLabel.setEnabled(true);

		instancePercentageFilterCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (instancePercentageFilterCheckBox.isSelected()) {
					instancePercentageFilterSlider.setEnabled(true);
					instancePercentageFilterValueLabel.setEnabled(true);
					minInstanceCountPercentageThreshold = instancePercentageFilterSlider.getValue();
				} else {
					instancePercentageFilterSlider.setEnabled(false);
					instancePercentageFilterValueLabel.setEnabled(false);
					minInstanceCountPercentageThreshold = 0;
				}
			}
		});

		instancePercentageFilterSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				instancePercentageFilterValueLabel.setText(instancePercentageFilterSlider.getValue() + "");
				if (!instancePercentageFilterSlider.getValueIsAdjusting()) {
					minInstanceCountPercentageThreshold = instancePercentageFilterSlider.getValue();
				}
			}
		});

		//Pattern Conservedness Parameter
		conservednessFilterCheckBox = new JCheckBox("Conservedness");
		conservednessFilterCheckBox.setOpaque(false);
		conservednessFilterCheckBox.setSelected(true);

		final JSlider conservednessFilterSlider = new JSlider(SwingConstants.HORIZONTAL, 0, 100, 0);
		conservednessFilterSlider.setUI(new SlickerSliderUI(conservednessFilterSlider));
		conservednessFilterSlider.setEnabled(true);
		d = conservednessFilterSlider.getPreferredSize();
		conservednessFilterSlider.setPreferredSize(new Dimension(d.width, d.height));
		//Set the initial value of the conservednessFilterSlider to the average conservedness
		conservednessFilterSlider.setValue(initalFilterPatternParameters[2]);
		conservednessFilterSlider
				.setToolTipText("<html>Filter pattern alphabets based on the conservedness metric which measures the degree<br>"
						+ "in which the individual activities involved in the pattern alphabet manifest as the patterns<br>"
						+ "defined by the alphabet.<br>"
						+ "Pattern alphabets below the threshold will be ignored.</html>");

		minConservednessThreshold = initalFilterPatternParameters[2];
		final JLabel conservednessFilterValueLabel = new JLabel("" + initalFilterPatternParameters[2] + "");
		conservednessFilterValueLabel.setEnabled(true);

		conservednessFilterCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (conservednessFilterCheckBox.isSelected()) {
					conservednessFilterSlider.setEnabled(true);
					conservednessFilterValueLabel.setEnabled(true);
					minConservednessThreshold = conservednessFilterSlider.getValue();
				} else {
					conservednessFilterSlider.setEnabled(false);
					conservednessFilterValueLabel.setEnabled(false);
					minConservednessThreshold = 0;
				}
			}
		});

		conservednessFilterSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				conservednessFilterValueLabel.setText(conservednessFilterSlider.getValue() + "");
				if (!conservednessFilterSlider.getValueIsAdjusting()) {
					minConservednessThreshold = conservednessFilterSlider.getValue();
				}
			}
		});

		//Pattern alphabet size parameter
		alphabetSizeFilterCheckBox = new JCheckBox("Alphabet Size");
		alphabetSizeFilterCheckBox.setOpaque(false);
		alphabetSizeFilterCheckBox.setSelected(true);

		final JSlider alphabetSizeFilterSlider = new JSlider(SwingConstants.HORIZONTAL, 0, 10, 0);
		alphabetSizeFilterSlider.setUI(new SlickerSliderUI(alphabetSizeFilterSlider));
		alphabetSizeFilterSlider.setEnabled(true);
		d = alphabetSizeFilterSlider.getPreferredSize();
		alphabetSizeFilterSlider.setPreferredSize(new Dimension(d.width, d.height));

		final JLabel alphabetSizeFilterValueLabel = new JLabel("0");
		alphabetSizeFilterValueLabel.setEnabled(true);
		alphabetSizeFilterSlider
				.setToolTipText("<html>Filter pattern alphabets based on the number of activities involved in the pattern alphabet.<br>"
						+ "Pattern alphabets below the threshold will be ignored.</html>");
		alphabetSizeFilterCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if (alphabetSizeFilterCheckBox.isSelected()) {
					alphabetSizeFilterSlider.setEnabled(true);
					alphabetSizeFilterValueLabel.setEnabled(true);
					minAlphabetSize = alphabetSizeFilterSlider.getValue();
				} else {
					alphabetSizeFilterSlider.setEnabled(false);
					alphabetSizeFilterValueLabel.setEnabled(false);
					minAlphabetSize = 0;
				}
			}
		});

		alphabetSizeFilterSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				alphabetSizeFilterValueLabel.setText(alphabetSizeFilterSlider.getValue() + "");
				if (!alphabetSizeFilterSlider.getValueIsAdjusting()) {
					minAlphabetSize = alphabetSizeFilterSlider.getValue();
				}
			}
		});

		//Pattern Frequency Panel
		JPanel ptFreqPanel = new JPanel();
		ptFreqPanel.setMaximumSize(new Dimension(800, 40));
		ptFreqPanel.setOpaque(false);
		ptFreqPanel.setLayout(new BoxLayout(ptFreqPanel, BoxLayout.X_AXIS));
		ptFreqPanel.add(Box.createHorizontalStrut(30));
		ptFreqPanel.add(patternFrequencyCheckBox);
		ptFreqPanel.add(Box.createHorizontalStrut(20));
		ptFreqPanel.add(patternFrequencySlider);
		ptFreqPanel.add(Box.createHorizontalStrut(10));
		ptFreqPanel.add(frequencyFilterValueLabel);

		//Pattern Instance Panel
		JPanel ptInstPanel = new JPanel();
		ptInstPanel.setMaximumSize(new Dimension(800, 40));
		ptInstPanel.setOpaque(false);
		ptInstPanel.setLayout(new BoxLayout(ptInstPanel, BoxLayout.X_AXIS));
		ptInstPanel.add(Box.createHorizontalStrut(30));
		ptInstPanel.add(instancePercentageFilterCheckBox);
		ptInstPanel.add(Box.createHorizontalStrut(20));
		ptInstPanel.add(instancePercentageFilterSlider);
		ptInstPanel.add(Box.createHorizontalStrut(10));
		ptInstPanel.add(instancePercentageFilterValueLabel);

		//Pattern Conservedness Panel
		JPanel ptConsPanel = new JPanel();
		ptConsPanel.setMaximumSize(new Dimension(800, 40));
		ptConsPanel.setOpaque(false);
		ptConsPanel.setLayout(new BoxLayout(ptConsPanel, BoxLayout.X_AXIS));
		ptConsPanel.add(Box.createHorizontalStrut(30));
		ptConsPanel.add(conservednessFilterCheckBox);
		//	ptConsPanel.add(Box.createHorizontalStrut(2));
		ptConsPanel.add(conservednessFilterSlider);
		ptConsPanel.add(Box.createHorizontalStrut(10));
		ptConsPanel.add(conservednessFilterValueLabel);

		//Pattern Alphabetsize Panel
		JPanel ptAlpPanel = new JPanel();
		ptAlpPanel.setMaximumSize(new Dimension(800, 40));
		ptAlpPanel.setOpaque(false);
		ptAlpPanel.setLayout(new BoxLayout(ptAlpPanel, BoxLayout.X_AXIS));
		ptAlpPanel.add(Box.createHorizontalStrut(30));
		ptAlpPanel.add(alphabetSizeFilterCheckBox);
		ptAlpPanel.add(Box.createHorizontalStrut(7));
		ptAlpPanel.add(alphabetSizeFilterSlider);
		ptAlpPanel.add(Box.createHorizontalStrut(10));
		ptAlpPanel.add(alphabetSizeFilterValueLabel);

		// assemble conf. panel
		JPanel patternCombPanel = new JPanel();
		patternCombPanel.setBackground(SlickerColors.COLOR_BG_2);
		patternCombPanel.setOpaque(false);
		JPanel p1 = new JPanel();
		String body = "<p>A Pattern refers to some common funtionality in a business process.</p>";
		body += "<p>This page is to guide you to choose the patterns to simplify the process model.</p>";
		body += "<p>Each parameter listed below is to measure the significance of the pattern.</p>";
		body += "<p>You can find the meaning of each parameter by putting the cursor on each slider.</p>";
		//body += "<p>We will use the value of each parameter to filter your desired pattern from the pattern sets in the log.</li>";
		body += "<p>The initial value is the average value of the total patterns.</p>";
		JLabel instructionLabel = SlickerFactory.instance().createLabel("<html><h1>Introduction</h1>" + body);
		p1.add(instructionLabel);
		p1.setOpaque(false);

		Box vbox = Box.createVerticalBox();
		//vbox.add(Box.createRigidArea(new Dimension(5, 20)));
		//vbox.add(Box.createRigidArea(new Dimension(5, 20)));
		vbox.add(p1);
		vbox.add(Box.createVerticalStrut(20));
		vbox.add(ptFreqPanel);
		vbox.add(Box.createVerticalStrut(20));
		vbox.add(ptInstPanel);
		vbox.add(Box.createVerticalStrut(20));
		vbox.add(ptConsPanel);
		vbox.add(Box.createVerticalStrut(20));
		vbox.add(ptAlpPanel);
		vbox.add(Box.createVerticalGlue());
		patternCombPanel.add(vbox, BorderLayout.CENTER);

		// add to root
		//GradientPanel back = new GradientPanel(new Color(80, 80, 80), new Color(40, 40, 40));
		//GradientPanel back = new GradientPanel(SlickerColors.COLOR_BG_1, SlickerColors.COLOR_BG_2);
		JPanel back = new JPanel();
		//	back.setLayout(new BorderLayout());
		//back.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		double size[][] = { { TableLayoutConstants.FILL }, { TableLayoutConstants.FILL } };
		back.setLayout(new TableLayout(size));
		//SlickerTabbedPane tabs = new SlickerTabbedPane("Configuration", new Color(20, 20, 20, 230), new Color(160, 160, 160, 180), new Color(220, 220, 220, 180));
		SlickerTabbedPane tabs = new SlickerTabbedPane("Configuration", SlickerColors.COLOR_BG_1,
				SlickerColors.COLOR_FG, SlickerColors.COLOR_FG);
		tabs.addTab("Pattern Filter", patternCombPanel);
		tabs.addTab("Measurement", rightSplit);
		tabs.addTab("Metrics", metricsPanel);
		//back.add(tabs, BorderLayout.CENTER);
		back.add(tabs, "0,0");
		back.setOpaque(true);
		//	HeaderBar header = new HeaderBar("Fuzzy Miner");
		//header.setHeight(40);

		//set the whole configuration panel
		//wholePanel.add(header, BorderLayout.NORTH);
		/*
		 * wholePanel.add(back, BorderLayout.CENTER);
		 * wholePanel.setBackground(new Color(40, 40, 40));
		 * wholePanel.setOpaque(true);
		 */
		return back;

	}

	protected void updateAttenuationPanel() {
		int maxDistance = maxRelDistSlider.getValue();
		if (attLinearRadioButton.isSelected()) {
			attenuation = new LinearAttenuation(maxDistance, maxDistance);
		} else if (attNRootRadioButton.isSelected()) {
			double n = attNRootNSlider.getValue() / 1000.0;
			attenuation = new NRootAttenuation(n, maxDistance);
		}
		attDisplayPanel.setAttenuation(attenuation);
		attDisplayPanel.setMaxDistance(maxDistance);
		attDisplayPanel.repaint();
	}

	public Object[] preprocess(XLog log) {

		//set Configuration UI
		//XLogInfo logsummary = XLogInfoFactory.createLogInfo(log);
		JPanel configPanel = getOptionsPanel();
		InteractionResult result = context.showConfiguration("Fuzzy Map Miner", configPanel);

		/*
		 * if(isCorrectlyConfigured() == false) { String message =
		 * "The Fuzzy Miner needs at least one active log-based metric\n" +
		 * "with a positive weight, from all major types, i.e.:\n" +
		 * " - at least one log-based unary metric\n" +
		 * " - at least one log-based binary significance metric\n" +
		 * " - at least one log-based binary correlation metric\n" +
		 * "Adjust your metrics' weight accordingly, please!";
		 * System.out.println(message); return null; }
		 */
		//Compute the Metrics
		switch (result) {
			case CANCEL :
				context.log("The user has cancelled the Fuzzy Map Miner!");
				context.getFutureResult(0).cancel(true);
				context.getFutureResult(1).cancel(true);
				context.getFutureResult(2).cancel(true);
				context.getFutureResult(3).cancel(true);
				context.getFutureResult(4).cancel(true);
				//	context.getFutureResult(6).cancel(true);
				return new Object[] { null, null, null, null, null };
			case CONTINUE :

				Set<Set<String>> filterPatternAlphabetSet = new HashSet<Set<String>>();
				System.out.println("The frequencyThreshold to use is " + minFrequencyThreshold);
				System.out.println("The instanceCount to use is " + minInstanceCountPercentageThreshold);
				System.out.println("The conservenessCount to use is " + minConservednessThreshold);
				System.out.println("The alphabetSize to use is " + minAlphabetSize);
				filterPatternAlphabetSet = ptnAbsFrm.getFilterPatternAlphabetSet(minFrequencyThreshold,
						minInstanceCountPercentageThreshold, minConservednessThreshold, minAlphabetSize);
				//get pattern tree which the label of each node is encoded
				PatternTree patternTree;
				String treeLabel = "Pattern Tree of the log " + XConceptExtension.instance().extractName(log);
				patternTree = getPatternTree(filterPatternAlphabetSet, treeLabel);
				//set the user configured mining parameters
				ArrayList<Object> miningParameters = new ArrayList<Object>();
				miningParameters.add(0, metrics);
				miningParameters.add(1, attenuation);
				miningParameters.add(2, new Integer(maxRelDistSlider.getValue()));
				//add all these parameters to the connection
				context.addConnection(new FuzzyMapConnection(patternTree, ptnAbsFrm, filterPatternAlphabetSet, log,
						miningParameters));
				return new Object[] { patternTree, ptnAbsFrm, filterPatternAlphabetSet, log, miningParameters };

			default :
				context.getFutureResult(0).cancel(true);
				context.getFutureResult(1).cancel(true);
				context.getFutureResult(2).cancel(true);
				context.getFutureResult(3).cancel(true);
				context.getFutureResult(4).cancel(true);
				//	context.getFutureResult(6).cancel(true);
				return new Object[] { null, null, null, null, null };
		}
	}

	/*
	 * use the filter pattern alphabet set to create the pattern tree
	 */
	public org.processmining.plugins.ptnBasedPreprocess.PatternTreeImpl getPatternTree(
			Set<Set<String>> filterPatternAlphabetSet, String treeLabel) {
		org.processmining.plugins.ptnBasedPreprocess.PatternTreeImpl patternTree = new PatternTreeImpl(treeLabel);
		//prepare the data structure for building the graph
		//according to the size of the alphabet to arrange the data structure
		sizeAlphabetSetMap = new HashMap<Integer, Set<PatternTreeNode>>();
		Set<PatternTreeNode> sizeAlphabetSet;
		//set the vertext of the Pattern Tree
		for (Set<String> alphabet : filterPatternAlphabetSet) {
			PatternTreeNode treeNode = patternTree.addNode(alphabet);
			if (sizeAlphabetSetMap.containsKey(alphabet.size())) {
				sizeAlphabetSet = sizeAlphabetSetMap.get(alphabet.size());
			} else {
				sizeAlphabetSet = new HashSet<PatternTreeNode>();
			}
			sizeAlphabetSet.add(treeNode);
			sizeAlphabetSetMap.put(alphabet.size(), sizeAlphabetSet);
		}

		// make an array of the different size of the filter pattern alphabet
		sortedSizeArray = new int[sizeAlphabetSetMap.size()];
		int index = 0;
		for (Integer size : sizeAlphabetSetMap.keySet()) {
			sortedSizeArray[index++] = size;
		}
		Arrays.sort(sortedSizeArray);
		buildPatternTree(patternTree);
		return patternTree;

	}

	protected void buildPatternTree(PatternTreeImpl patternTree) {
		int maxStepSize = sortedSizeArray.length - 1;
		int lowerLayerAlphabetSize, upperLayerAlphabetSize;
		Set<PatternTreeNode> lowerLayerPatternNodeSet, upperLayerPatternNodeSet;
		Set<String> lowerLayerPatternLabel, upperLayerPatternLabel;
		for (int step = 1; step <= maxStepSize; step++) {
			System.out.println("Step : " + step);
			for (int i = 0; i < sortedSizeArray.length - step; i++) {
				System.out.println("\tLayer " + i + ", " + (i + step));
				lowerLayerAlphabetSize = sortedSizeArray[i];
				upperLayerAlphabetSize = sortedSizeArray[i + step];

				lowerLayerPatternNodeSet = sizeAlphabetSetMap.get(lowerLayerAlphabetSize);
				upperLayerPatternNodeSet = sizeAlphabetSetMap.get(upperLayerAlphabetSize);
				for (PatternTreeNode alphabetI : lowerLayerPatternNodeSet) {
					for (PatternTreeNode alphabetJ : upperLayerPatternNodeSet) {
						lowerLayerPatternLabel = FuzzyMapUtil.getActivityLabelsInPattern(alphabetI, encodingLength);
						upperLayerPatternLabel = FuzzyMapUtil.getActivityLabelsInPattern(alphabetJ, encodingLength);
						if (upperLayerPatternLabel.containsAll(lowerLayerPatternLabel)) {
							ShortestPathInfo<PatternTreeNode, PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> shortestPathInfo = ShortestPathFactory
									.calculateAllShortestDistanceDijkstra(patternTree);
							if (shortestPathInfo.getShortestPathLength(alphabetI, alphabetJ) < 0) {
								patternTree.addArc(alphabetI, alphabetJ);
							}
						}
					}
				}
			}
		}

		/*
		 * if the indegree of the pattern node which size is larger than 1 , add
		 * the original activity nodes as its input nodes and add the edges
		 * between them
		 */
		Set<PatternTreeNode> largerPatternNodeSet;
		Set<String> activitiesInPattern;
		for (Integer i : sizeAlphabetSetMap.keySet()) {
			if (i > 1) {
				largerPatternNodeSet = sizeAlphabetSetMap.get(i);
				for (PatternTreeNode node : largerPatternNodeSet) {
					activitiesInPattern = FuzzyMapUtil.getActivityLabelsInPattern(node, encodingLength);
					Set<PatternTreeNode> predecessors = node.getPredecessors();
					Set<String> labelsOfPreNodes = new HashSet<String>();
					for (PatternTreeNode existPreNode : predecessors) {
						String precessorLabel;
						precessorLabel = existPreNode.getLabel();
						Set<String> activityNameInPrecessorNode = FuzzyMapUtil.getOriginalActivityNameInPattern(
								precessorLabel, encodingLength);
						for (String activityName : activityNameInPrecessorNode) {
							labelsOfPreNodes.add(activityName);
						}
					}
					if (labelsOfPreNodes.size() < activitiesInPattern.size()) {
						//get every predecessor node's name
						activitiesInPattern.removeAll(labelsOfPreNodes);
						for (String activityName : activitiesInPattern) {
							String labelOfNodeToAdd = "[" + activityName + "]";
							TreeSet<String> labelAlphabet = new TreeSet<String>();
							labelAlphabet.add(activityName);
							PatternTreeNode originalNode = patternTree.containNode(labelOfNodeToAdd);
							if (originalNode != null) {
								patternTree.addArc(originalNode, node);
							} else {
								PatternTreeNode sourceNode = patternTree.addNode(labelAlphabet);
								patternTree.addArc(sourceNode, node);
								//set the artificial pattern's count to the number of the pattern count which subsume this artificial pattern
								Integer sourceNodeCount = ptnAbsFrm.getNonOverlapAlphabetCountMap().get(
										node.getLabelAlphabet());
								ptnAbsFrm.getNonOverlapAlphabetCountMap().put(labelAlphabet, sourceNodeCount);
							}
						}
					}
				}
			}

		}

	}

	public void setSizeAlphabetSetMap() {

	}

	protected boolean isCorrectlyConfigured() {
		boolean minOneUnary = false;
		boolean minOneBinarySig = false;
		for (UnaryMetric metric : metrics.getUnaryLogMetrics()) {
			if (metric.getNormalizationMaximum() > 0.0) {
				minOneUnary = true;
				break;
			}
		}
		if (minOneUnary == false) {
			return false;
		}
		for (BinaryMetric metric : metrics.getSignificanceBinaryLogMetrics()) {
			if (metric.getNormalizationMaximum() > 0.0) {
				minOneBinarySig = true;
				break;
			}
		}
		if (minOneBinarySig == false) {
			return false;
		}
		for (BinaryMetric metric : metrics.getCorrelationBinaryLogMetrics()) {
			if (metric.getNormalizationMaximum() > 0.0) {
				return true;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.framework.plugin.Plugin#getHtmlDescription()
	 */
	public String getHtmlDescription() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.framework.plugin.Plugin#getName()
	 */
	public String getName() {
		return "Fuzzy Map Miner";
	}

}
