package org.processmining.plugins.fuzzymodel;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.plugin.events.Logger.MessageLevel;
import org.processmining.models.connections.fuzzymodel.FuzzyModelConnection;
import org.processmining.models.graphbased.directed.fuzzymodel.MutableFuzzyGraph;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.MetricsRepository;
import org.processmining.plugins.fuzzymodel.miner.ui.FuzzyModelInformationPanel;

/**
 * Visualization support for Fuzzy Model.  * 
 * @author Jiafei Li (jiafei@jlu.edu.cn)
 * 
 */
@Plugin(name = "Fuzzy Model visualization", 
		returnLabels = { "Visualized Fuzzy Model" }, 
		returnTypes = { JComponent.class }, 
		parameterLabels = {"Fuzzy Model"}, 
		userAccessible = false)
@Visualizer
public class FuzzyModelVisualization {
	@PluginVariant(requiredParameterLabels = {0})
	public JComponent visualize(PluginContext context, 	MutableFuzzyGraph graph){
		try {
			ConnectionManager cm = context.getConnectionManager();
     		if (cm.getConnections(FuzzyModelConnection.class, context) != null){
				MetricsRepository metrics = graph.getMetrics();
				return getVisualizationPanel(context, graph,metrics);
			}
			else
			{
				return null;
			}
		} catch (ConnectionCannotBeObtained e) {
			// No connections available
			context.log("Connection is not exist", MessageLevel.DEBUG);
			getErrorMessagePanel("No connections available");
			return null;
			
		}

		
	}


	
	/**
	 * This method only invoked after all connection checking is performed
	 * @param context
	 * @param graph
	 * @param log
	 * @param metrics
	 * @param numberOfNodes
	 * @return
	 */
	
	private JComponent getVisualizationPanel(PluginContext context, MutableFuzzyGraph graph, MetricsRepository metrics) {
		//System.out.println("Start to Get the Fuzzy Model VisualizationPanel!!!");
		int maxNumOfInitialPrimitiveNodes = 7;
		FuzzyModelInformationPanel fmPanel = new FuzzyModelInformationPanel(context,graph, context.getProgress(),
				                                             graph.getLog(), metrics, maxNumOfInitialPrimitiveNodes);
			
		return fmPanel;
	}
	
	/**
	 * return error message panel
	 *  
	 */
	private JComponent getErrorMessagePanel(String message) {
		JLabel errorMessage = new JLabel();
		errorMessage.setText(message);

		JPanel panel = new JPanel();
		panel.add(errorMessage);

		return panel;
	}
}
