/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which are not
 * licensed under the terms of the GPL, given that they satisfy one or more of
 * the following conditions: 1) Explicit license is granted to the ProM and
 * ProMimport programs for usage, linking, and derivative work. 2) Carte blance
 * license is granted to all programs developed at Eindhoven Technical
 * University, The Netherlands, or under the umbrella of STW Technology
 * Foundation, The Netherlands. For further exemptions not covered by the above
 * conditions, please contact the author of this code.
 */
package org.processmining.plugins.fuzzymodel.miner.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.deckfour.xes.model.impl.XAttributeTimestampImpl;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.fuzzymodel.MutableFuzzyGraph;
import org.processmining.plugins.fuzzymodel.miner.filter.FMEventEditor;
import org.processmining.plugins.fuzzymodel.miner.filter.FMEventTimeInjectionFilter;
import org.processmining.plugins.fuzzymodel.miner.filter.FMTraceEditor;
import org.processmining.plugins.fuzzymodel.miner.vis.ui.AnimationFrame;
import org.processmining.plugins.log.logfilters.LogFilter;
import org.processmining.plugins.log.logfilters.LogFilterException;

import com.fluxicon.slickerbox.components.AutoFocusButton;
import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.ui.SlickerCheckBoxUI;
import com.fluxicon.slickerbox.ui.SlickerComboBoxUI;

/**
 * @author Christian W. Guenther (christian@deckfour.org)
 * @author Jiafei Li(jiafei@jlu.edu.cn)
 */
public class FuzzyAnimationPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3739824678771086023L;
	protected MutableFuzzyGraph graph = null;
	protected XLog log = null;
	protected boolean discreteAnimation = false;
	protected JComboBox logSelector;
	protected NiceIntegerSlider lookaheadSlider;
	protected NiceIntegerSlider extraLookaheadSlider;
	protected PluginContext context;

	protected AnimationFrame animationFrame = null;

	protected HashMap<String, XLog> logMap;

	public FuzzyAnimationPanel(PluginContext context, MutableFuzzyGraph graph, XLog log) {
		this.context = context;
		logMap = new HashMap<String, XLog>();
		String logname = ((XAttributeLiteralImpl)log.getAttributes().get("concept:name")).getValue();
		logMap.put(logname , log);
		this.addAncestorListener(new AncestorListener() {
			public void ancestorAdded(AncestorEvent event) {
	//			updateResources();
				logSelector.setModel(new DefaultComboBoxModel(logMap.keySet().toArray()));
			}

			public void ancestorMoved(AncestorEvent event) { //ignore 
			}

			public void ancestorRemoved(AncestorEvent event) { // ignore 
			}
		});

		setLog(log);
		//updateResources();
		this.graph = graph;
		discreteAnimation = false;
		animationFrame = null;
		// setup UI
		this.setOpaque(false);
		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		this.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
		RoundedPanel innerPanel = new RoundedPanel(20);
		innerPanel.setAlignmentY(JPanel.CENTER_ALIGNMENT);
		innerPanel.setLayout(new BoxLayout(innerPanel, BoxLayout.Y_AXIS));
		innerPanel.setBackground(new Color(140, 140, 140));
		innerPanel.setMinimumSize(new Dimension(350, 250));
		innerPanel.setMaximumSize(new Dimension(350, 250));
		innerPanel.setPreferredSize(new Dimension(350, 250));
		final JCheckBox discreteAnimBox = new JCheckBox("discrete animation (inject timestamps)");
		discreteAnimation = false;
		if (log != null) {
			try {
				//discreteAnimation = (log.getInstance(0).getAuditTrailEntryList().get(0).getTimestamp() == null);
				XEvent firstEvent = log.get(0).get(0);
				Date evttime;
				if (firstEvent.getAttributes().containsKey("time:timestamp"))
				{
					evttime = ((XAttributeTimestampImpl) firstEvent.getAttributes().get("time:timestamp")).getValue();
				}
				else{
					evttime = null;
					//inject time stamp to the log for replaying
					FMEventTimeInjectionFilter tsFilter = new FMEventTimeInjectionFilter();
					for (int i = 0; i < log.size(); i++) {
						XTrace curTrace = log.get(i);
						tsFilter.doFiltering(curTrace);
					}
				}
				discreteAnimation = (evttime == null);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		discreteAnimBox.setAlignmentX(JCheckBox.CENTER_ALIGNMENT);
		discreteAnimBox.setSelected(discreteAnimation);
		discreteAnimBox.setForeground(new Color(40, 40, 40));
		discreteAnimBox.setUI(new SlickerCheckBoxUI());
		discreteAnimBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				discreteAnimation = discreteAnimBox.isSelected();
			}
		});
		JButton animButton = new AutoFocusButton("view animation");
		animButton.setAlignmentX(JButton.CENTER_ALIGNMENT);
		animButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				createAnimation();
			}
		});
		// graph and log selector boxes
		logSelector = new JComboBox(logMap.keySet().toArray());
		logSelector.setUI(new SlickerComboBoxUI());
		// preselect configured objects
		if (log == null && logSelector.getItemCount() > 0) {
			logSelector.setSelectedIndex(0);
			setLog(logMap.get(logSelector.getSelectedItem()));
		}
		for (String logKey : logMap.keySet()) {
			if (logMap.get(logKey) == log) {
				logSelector.setSelectedItem(logKey);
				break;
			}
		}
		// add listeners
		logSelector.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				setLog(logMap.get(logSelector.getSelectedItem()));
			}
		});
		// labels
		JLabel graphLabel = new JLabel("Select Fuzzy Model to animate:");
		graphLabel.setOpaque(false);
		graphLabel.setForeground(new Color(30, 30, 30));
		graphLabel.setAlignmentX(JLabel.CENTER_ALIGNMENT);
		JLabel logLabel = new JLabel("Select Log for animation:");
		logLabel.setOpaque(false);
		logLabel.setForeground(new Color(30, 30, 30));
		logLabel.setAlignmentX(JLabel.CENTER_ALIGNMENT);
		// lookahead sliders
		lookaheadSlider = new NiceIntegerSlider("Lookahead", 1, 25, 5);
		lookaheadSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				int max = lookaheadSlider.getValue() - 1;
				extraLookaheadSlider.getSlider().setMaximum(max);
				if (extraLookaheadSlider.getValue() > max) {
					extraLookaheadSlider.setValue(max);
				}
			}
		});
		extraLookaheadSlider = new NiceIntegerSlider("Extra lookahead", 0, 15, 3);
		// assemble GUI
		innerPanel.add(Box.createVerticalStrut(15));
		innerPanel.add(logLabel);
		innerPanel.add(Box.createVerticalStrut(8));
		innerPanel.add(logSelector);
		innerPanel.add(Box.createVerticalStrut(15));
		innerPanel.add(discreteAnimBox);
		innerPanel.add(Box.createVerticalStrut(15));
		innerPanel.add(lookaheadSlider);
		innerPanel.add(Box.createVerticalStrut(8));
		innerPanel.add(extraLookaheadSlider);
		innerPanel.add(Box.createVerticalStrut(15));
		innerPanel.add(animButton);
		innerPanel.add(Box.createVerticalStrut(15));
		this.add(Box.createHorizontalGlue());
		this.add(innerPanel);
		this.add(Box.createHorizontalGlue());
	}

	public void setGraph(MutableFuzzyGraph graph) {
		this.graph = graph;
	}

	public void setLog(XLog log) {
		if (log != null) {
			this.log = log;
		}
	}

	protected void createAnimation() {

		FMEventEditor fmEventEditor = new FMEventEditor(context, graph);
		FMTraceEditor fmTraceEditor = new FMTraceEditor();
		try {
			XLog filteredLog = LogFilter.filter(context.getProgress(),1, log, null, fmEventEditor, fmTraceEditor);
			if (discreteAnimation == true) {
				FMEventTimeInjectionFilter tsFilter = new FMEventTimeInjectionFilter();
			//	XLog timeFilteredLog = (XLog)filteredLog.clone();
				Iterator<XTrace> it = filteredLog.iterator();
				while (it.hasNext()) {
					XTrace oldTrace = it.next();
					tsFilter.doFiltering(oldTrace);
//					Date start = FuzzyMinerLog.getEventTime(oldTrace.get(0));
//					Date end = FuzzyMinerLog.getEventTime(oldTrace.get(oldTrace.size()-1));
				}
			    
				
			/*	for (int i = 0; i < filteredLog.size(); i++) {
					XTrace curTrace = filteredLog.get(i);
					XTrace filteredTrace = tsFilter.doFiltering(curTrace);
					Date start = FuzzyMinerLog.getEventTime(filteredTrace.get(0));
					Date end = FuzzyMinerLog.getEventTime(filteredTrace.get(curTrace.size()-1));
					timeFilteredLog.add(filteredTrace);
				}
				filteredLog = timeFilteredLog;*/
			}
			
			animationFrame = new AnimationFrame(context, filteredLog, graph, lookaheadSlider.getValue(),
					extraLookaheadSlider.getValue());
			JFrame frame = new JFrame("Fuzzy Model Animation");
			frame.setContentPane(animationFrame);
			Toolkit kit = Toolkit.getDefaultToolkit();
			Dimension screenSize = kit.getScreenSize();
			int screenHeight = screenSize.height;
			int screenWidth =  screenSize.width;
			frame.setSize(3*screenWidth/4,3*screenHeight/4);
			frame.setLocation(screenWidth/8, screenHeight/8);
			frame.setVisible(true);
			//frame.setLocation(50, 50);
			//frame.setSize(1000, 600);
			
			//		frame.pack();
			//MainUI.getInstance().createFrame("Fuzzy Model Animation", animationFrame);		
		} catch (LogFilterException filterException) {
			filterException.printStackTrace();
		}
		 catch (Exception e1) {
				e1.printStackTrace();
		}
	}

/*	protected void updateResources() {
		logMap = new HashMap<String, XLog>();
		JInternalFrame[] frames = MainUI.getInstance().getDesktop().getAllFrames();
		for (JInternalFrame frame : frames) {
			if (frame instanceof Provider) {
				ProvidedObject[] providedObjects = ((Provider) frame).getProvidedObjects();
				for (ProvidedObject providedObject : providedObjects) {
					for (Object object : providedObject.getObjects()) {
						if (object instanceof XLog) {
							logMap.put(frame.getTitle() + " - " + providedObject.getName(), (XLog) object);
						}
					}
				}
			}
		}
	}*/

}
