package org.processmining.plugins.fuzzymodel.miner.ui;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.fuzzymodel.FuzzyGraph;
import org.processmining.models.graphbased.directed.fuzzymodel.MutableFuzzyGraph;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.MetricsRepository;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.binary.BinaryMetric;
import org.processmining.models.graphbased.directed.fuzzymodel.metrics.unary.UnaryMetric;
import org.processmining.models.graphbased.directed.fuzzymodel.util.FuzzyMinerLog;
import org.processmining.models.graphbased.directed.jgraph.JGraphVisualizationPanel;
import org.processmining.plugins.fuzzymodel.miner.edit.FuzzyGraphEditorPanel;

import com.fluxicon.slickerbox.colors.SlickerColors;
import com.fluxicon.slickerbox.components.GradientPanel;
import com.fluxicon.slickerbox.components.HeaderBar;
import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;

public class FuzzyModelInformationPanel extends GradientPanel{//extends GraphInformationPanel {//implements Provider {

	/**
	 * 
	 */
	private static final long serialVersionUID = -3186276401263823237L;


	// general GUI components
	
	
	//protected FuzzyGraph originalGraph;
	protected MutableFuzzyGraph graph;
	protected PluginContext context;
	protected Progress progress;
	protected XLog log;
	protected MetricsRepository metrics;
	protected Color background = new Color(100, 100, 100);
	protected Color canvasBackground = Color.WHITE;
	protected int numberOfNodes;
	
	protected FastTransformerPanel fastTransformerPanel;
	protected FuzzyGraphEditorPanel editorPanel;
	protected UnaryMetricsPanel unaryMetricsPanel;
	protected BinaryMetricsComparisonPanel binaryMetricsComparisonPanel;
	protected FuzzyAnimationPanel animationPanel;

	protected JGraphVisualizationPanel graphVisPanel;
	protected ViewSpecificAttributeMap viewSpecificMap;
//	protected JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true);
	

	public FuzzyModelInformationPanel(PluginContext context, MutableFuzzyGraph graph, Progress progress,XLog log, MetricsRepository metrics, int numberOfNodes) {
		//super(graph, new ViewSpecificAttributeMap(), progress);
		// General layout
		super(new Color(60,60,60), new Color(20,20,20));
		setLayout(new BorderLayout());

		this.viewSpecificMap = new ViewSpecificAttributeMap();
		this.progress = progress;
		this.context = context;

	//	this.originalGraph = graph;
		this.graph = graph;
		this.graph.setNodeSignificance(new UnaryMetric("Node significance", "Node significance", graph.getNodeSignificanceMetric()));
		this.graph.setEdgeSignificance(new BinaryMetric("Edge significance", "Edge significance", graph.getEdgeSignificanceMetric()));
		this.graph.setEdgeCorrelation(new BinaryMetric("Edge correlation", "Edge correlation", graph.getEdgeCorrelationMetric()));
		
		this.log = log;
		this.metrics = metrics;
		this.numberOfNodes = numberOfNodes;
		setupUI();
	}
	
	

	protected void setupUI() {
		// setup panel basics
		this.setDoubleBuffered(true);
		this.setLayout(new BorderLayout());
		this.setBorder(BorderFactory.createEmptyBorder());
		//FlatTabbedPane tabPane = new FlatTabbedPane("", new Color(240, 240, 240, 230), 
			//	new Color(100, 100, 100), new Color(220, 220, 220, 150));
		SlickerTabbedPane tabPane = new SlickerTabbedPane("", SlickerColors.COLOR_BG_1, SlickerColors.COLOR_FG, SlickerColors.COLOR_FG);
		fastTransformerPanel = new FastTransformerPanel(context,graph, log, numberOfNodes);
		tabPane.addTab("Transformer", fastTransformerPanel);
	
//		editorPanel = new FuzzyGraphEditorPanel(context,graph, false);
//		tabPane.addTab("Editor", editorPanel);
		if(metrics != null) {
			// create unary metrics comparison panel
			unaryMetricsPanel = 
				new UnaryMetricsPanel(FuzzyMinerLog.getLogEvents(metrics.getLogReader()));
			for(UnaryMetric unary : metrics.getUnaryMetrics()) {
				unaryMetricsPanel.addMetric(unary);
			}
			unaryMetricsPanel.addMetric(metrics.getAggregateUnaryMetric());
			RoundedPanel unaryMetricsEnclosure = new RoundedPanel(10, 5, 0);
			unaryMetricsEnclosure.setBackground(Color.black);
			unaryMetricsEnclosure.setLayout(new BorderLayout());
			unaryMetricsEnclosure.add(unaryMetricsPanel, BorderLayout.CENTER);
			// create binary metrics comparison panel
			binaryMetricsComparisonPanel = new BinaryMetricsComparisonPanel(metrics);
			// add to tabbed pane
			tabPane.addTab("Unary Metrics", unaryMetricsEnclosure);
			tabPane.addTab("Binary Metrics", binaryMetricsComparisonPanel);
			animationPanel = new FuzzyAnimationPanel(context,graph, log);
			tabPane.addTab("Animation", animationPanel);
		}		
		// assemble UI
		JPanel centerPanel = new JPanel();
		centerPanel.setOpaque(false);
		centerPanel.setLayout(new BorderLayout());
		centerPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		centerPanel.add(tabPane, BorderLayout.CENTER);
		this.add(centerPanel, BorderLayout.CENTER);
		HeaderBar header = new HeaderBar("FM (mined from " +  XConceptExtension.instance().extractName(log) + ")");
		header.setHeight(40);
		this.add(header, BorderLayout.NORTH);
	}
	
/*	public FuzzyGraph getOriginalGraph() {
		return originalGraph;
	}*/
	
	public FuzzyGraph getCurrentGraph() {
		return graph;
	}

	public XLog getUsedLog(){
		return log;
	}
	
	public PluginContext getContext(){
		return context;
	}
	/* (non-Javadoc)
	 * @see org.processmining.framework.plugin.Provider#getProvidedObjects()
	 */
	/*public ProvidedObject[] getProvidedObjects() {
		ProvidedObject[] ftpo = fastTransformerPanel.getProvidedObjects();
		ProvidedObject[] objects = new ProvidedObject[ftpo.length + 1];
		for(int i=0; i<ftpo.length; i++) {
			objects[i] = ftpo[i];
		}
		objects[ftpo.length] = new ProvidedObject("Original Fuzzy Model", new Object[] { originalGraph });
		return objects;
	}*/


}
