/*
 * Copyright (c) 2007 Christian W. Guenther (christian@deckfour.org)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * 
 * EXEMPTION:
 * 
 * License to link and use is also granted to open source programs which
 * are not licensed under the terms of the GPL, given that they satisfy one
 * or more of the following conditions:
 * 1) Explicit license is granted to the ProM and ProMimport programs for
 *    usage, linking, and derivative work.
 * 2) Carte blance license is granted to all programs developed at
 *    Eindhoven Technical University, The Netherlands, or under the
 *    umbrella of STW Technology Foundation, The Netherlands.
 * For further exemptions not covered by the above conditions, please
 * contact the author of this code.
 * 
 */
package org.processmining.plugins.fuzzymodel.miner.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JViewport;

import org.processmining.plugins.fuzzymodel.miner.replay.FuzzyReplay;

import com.fluxicon.slickerbox.components.HeaderBar;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

/**
 * @author christian
 *
 */
public class ReplayView extends JPanel {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -8930350118686109975L;
	protected FuzzyReplay replay;
	protected HeaderBar headerBar;
	
	public ReplayView(FuzzyReplay replay) {
		this.replay = replay;
		this.setBorder(BorderFactory.createEmptyBorder());
		this.setLayout(new BorderLayout());
		JPanel tracePanel = new JPanel();
		tracePanel.setLayout(new BoxLayout(tracePanel, BoxLayout.Y_AXIS));
		tracePanel.setBorder(BorderFactory.createEmptyBorder());
		tracePanel.setOpaque(true);
		tracePanel.setBackground(new Color(30, 30, 30));
		tracePanel.add(Box.createVerticalStrut(10));
		// add all trace replay views
		for(int i=0; i<replay.numberOfTraces(); i++) {
			TraceReplayView traceView = new TraceReplayView(replay.getTraceReplay(i));
			traceView.setAlignmentX(LEFT_ALIGNMENT);
			JPanel comprisePanel = new JPanel();
			comprisePanel.setAlignmentX(LEFT_ALIGNMENT);
			comprisePanel.setBorder(BorderFactory.createEmptyBorder());
			comprisePanel.setOpaque(false);
			comprisePanel.setLayout(new BoxLayout(comprisePanel, BoxLayout.X_AXIS));
			comprisePanel.add(traceView);
			comprisePanel.add(Box.createHorizontalGlue());
			tracePanel.add(comprisePanel);
		}
		JScrollPane scrollPane = new JScrollPane(tracePanel);
		scrollPane.getVerticalScrollBar().setUnitIncrement(40);
		scrollPane.getHorizontalScrollBar().setUnitIncrement(40);
		scrollPane.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setBackground(new Color(30, 30, 30));
		scrollPane.setOpaque(true);
		JScrollBar vBar = scrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(30, 30, 30), new Color(140, 140, 140), new Color(80, 80, 80), 4, 12));
		JScrollBar hBar = scrollPane.getHorizontalScrollBar();
		hBar.setUI(new SlickerScrollBarUI(hBar, new Color(30, 30, 30), new Color(140, 140, 140), new Color(80, 80, 80), 4, 12));
		headerBar = new HeaderBar("trace conformance inspector");
		this.add(headerBar, BorderLayout.NORTH);
		this.add(scrollPane, BorderLayout.CENTER);
	}
	
	public void setCloseActionListener(ActionListener listener) {
		this.headerBar.setCloseActionListener(listener);
	}
	

}
