package org.processmining.plugins.ptnBasedPreprocess;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.plugin.events.Logger.MessageLevel;
import org.processmining.plugins.ptnBasedPreprocess.ui.PatternGraphUI;

/**
 * Visualization support for Fuzzy Map. *
 * 
 * @author Jiafei Li (jiafei@jlu.edu.cn)
 * 
 */

@Plugin(name = "Pattern Graph Panel Visualization", returnLabels = { "Visualizion of Pattern Based Abstractions" }, returnTypes = { JComponent.class }, parameterLabels = { "PatternGraphUI" }, userAccessible = false)
@Visualizer
public class PatternBasedPreprocessVisulization {
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, PatternGraphUI graphUI) {
		ConnectionManager cm = context.getConnectionManager();
		try {
			if (cm.getConnections(PatternBasedPreprocessConnection.class, context) != null) {
				return graphUI;
			}
			return null;
		} catch (ConnectionCannotBeObtained e) {
			// No connections available
			context.log("Connection does not exist", MessageLevel.DEBUG);
			return null;
		}
	}
}

//The following code is for improving plug-in
/*
 * @Plugin(name = "Pattern Graph visualization", returnLabels = {
 * "Visualized Pattern Graph" }, returnTypes = { JComponent.class },
 * parameterLabels = {"Pattern Graph", "Pattern Abstraction Frame",
 * "filter Alphabet Set", "log"}, userAccessible = false)
 * 
 * @Visualizer public class PatternBasedPreprocessVisulization {
 * 
 * @PluginVariant(requiredParameterLabels = {0}) public JComponent
 * visualize(UIPluginContext context,PatternTree patternTree) {
 * System.out.println("PatternBasedPreprocessVisulization Start!!!");
 * System.out.println("The PatternBasedPreprocessVisulization context is " +
 * context.toString()); try {
 * System.out.println("Start to get PatternBasedPreprocessVisulization!");
 * ConnectionManager cm = context.getConnectionManager();
 * PatternBasedPreprocessConnection fmConn =
 * cm.getFirstConnection(PatternBasedPreprocessConnection.class, context);
 * System.out.println("have got the  PatternBasedPreprocessVisulization");
 * 
 * // connection found. Create all necessary component to instantiate inactive
 * visualization panel PatternAbstractionFrame ptnAbsFrm =
 * fmConn.getObjectWithRole
 * (PatternBasedPreprocessConnection.PATTERN_ABSTRACTION_FRAME);
 * Set<Set<String>> filterPatternAlphabetSet =
 * fmConn.getObjectWithRole(PatternBasedPreprocessConnection
 * .FILTERED_PATTERN_ALPHABETSET); XLog log =
 * fmConn.getObjectWithRole(PatternBasedPreprocessConnection.LOG); return
 * getVisualizationPanel(context,
 * patternTree,ptnAbsFrm,filterPatternAlphabetSet,log); } catch
 * (ConnectionCannotBeObtained e) { // No connections available
 * context.log("Connection is not exist", MessageLevel.DEBUG);
 * getErrorMessagePanel("No connections available"); return null;
 * 
 * }
 * 
 * 
 * }
 */

/**
 * This method only invoked after all connection checking is performed
 * 
 * @param context
 * @param graph
 * @param log
 * @param metrics
 * @param numberOfNodes
 * @return
 */
/*
 * private JComponent getVisualizationPanel(UIPluginContext context, PatternTree
 * patternTree, PatternAbstractionFrame ptnAbsFrm, Set<Set<String>>
 * filterPatternAlphabetSet, XLog log) {
 * System.out.println("Start to Get the Abstraction graph VisualizationPanel!!!"
 * ); PatternGraphUI ptnGraphPanel = new
 * PatternGraphUI(context,patternTree,ptnAbsFrm,filterPatternAlphabetSet,log);
 * return ptnGraphPanel; }
 */

/**
 * return error message panel
 * 
 */
/*
 * private JComponent getErrorMessagePanel(String message) { JLabel errorMessage
 * = new JLabel(); errorMessage.setText(message);
 * 
 * JPanel panel = new JPanel(); panel.add(errorMessage);
 * 
 * return panel; } }
 */

