package org.processmining.plugins.ptnBasedPreprocess;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.SwingConstants;

import org.jgraph.graph.DefaultEdge;
import org.jgrapht.graph.SimpleDirectedGraph;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.graphbased.directed.patterntree.PatternTree;
import org.processmining.models.graphbased.directed.patterntree.PatternTreeEdge;
import org.processmining.models.graphbased.directed.patterntree.PatternTreeNode;
import org.processmining.models.graphbased.directed.patterntree.PatternTreeUtil;
import org.processmining.models.graphbased.directed.patterntree.elements.PatternTreeArcElement;
import org.processmining.models.graphbased.directed.patterntree.elements.PatternTreeNodeElement;
import org.processmining.plugins.abstractions.PatternAbstractionFrame;
import org.processmining.plugins.fuzzymap.util.FuzzyMapUtil;
import org.processmining.plugins.ptnBasedPreprocess.util.PatternAbstractionFrameUtil;

/**
 * @author Jiafei Li
 * @email jiafei@jlu.edu.cn
 * @version Mar 8, 2010
 */
public class PatternTreeImpl extends
		AbstractDirectedGraph<PatternTreeNode, PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>>
		implements PatternTree {

	protected Set<PatternTreeNode> nodes;
	protected Set<PatternTreeEdge<PatternTreeNode, PatternTreeNode>> arcs;
	//Map between single activity and its count
	protected Map<String, Integer> encodedActivityCountMap;
	//protected PatternAbstractionFrame ptnAbsFrm;
	protected PatternAbstractionFrameUtil ptnAbsFrm;
	//Map between the pattern (encoded) and the  count of its total occurrence in the non-overlapping condition
	protected Map<TreeSet<String>, Integer> actualNonOverlapAlphabetCountMap;
	//Map between the pattern (encoded) and the count of its isolation occurrence in the non-overlapping condition
	protected Map<TreeSet<String>, Integer> patternIsolationNonOverlapAlphabetCountMap;
	//conservedness
	protected Map<TreeSet<String>, Integer> actualNonOverlapAlphabetConservednessMetricMap;
	protected int encodingLength;
	protected Map<Integer, Set<PatternTreeNode>> sizeAlphabetSetMap;
	protected int[] sortedSizeArray;

	public PatternTreeImpl(Set<Set<String>> filterPatternAlphabetSet, Set<Set<String>> notUsedSingleActivitySet,
			Map<String, Integer> encodedActivityCountMap,
			Map<TreeSet<String>, Integer> actualNonOverlapAlphabetCountMap,
			Map<TreeSet<String>, Integer> patternIsolationNonOverlapAlphabetCountMap,
			Map<TreeSet<String>, Integer> actualNonOverlapAlphabetConservednessMetricMap,
			PatternAbstractionFrameUtil ptnAbsFrm, String label, int encodingLength) {
		super();

		getAttributeMap().put(AttributeMap.PREF_ORIENTATION, SwingConstants.WEST);
		//	getAttributeMap().put(AttributeMap.
		this.encodingLength = encodingLength;
		getAttributeMap().put(AttributeMap.LABEL, label);
		nodes = new LinkedHashSet<PatternTreeNode>();
		arcs = new LinkedHashSet<PatternTreeEdge<PatternTreeNode, PatternTreeNode>>();
		this.ptnAbsFrm = ptnAbsFrm;
		this.encodedActivityCountMap = encodedActivityCountMap;
		this.actualNonOverlapAlphabetCountMap = actualNonOverlapAlphabetCountMap;
		this.patternIsolationNonOverlapAlphabetCountMap = patternIsolationNonOverlapAlphabetCountMap;
		this.actualNonOverlapAlphabetConservednessMetricMap = actualNonOverlapAlphabetConservednessMetricMap;
		initializeGraph(filterPatternAlphabetSet, notUsedSingleActivitySet);

	}

	public PatternTreeImpl(String label) {
		super();
		getAttributeMap().put(AttributeMap.LABEL, label);
		nodes = new LinkedHashSet<PatternTreeNode>();
		arcs = new LinkedHashSet<PatternTreeEdge<PatternTreeNode, PatternTreeNode>>();

	}

	public void initializeGraph(Set<Set<String>> filterPatternAlphabetSet, Set<Set<String>> notUsedSingleActivitySet) {
		//delete the elements of graph first in order to redraw the graph
		for (PatternTreeEdge<PatternTreeNode, PatternTreeNode> edge : arcs) {
			graphElementRemoved(edge);
		}
		for (PatternTreeNode node : nodes) {
			if (node != null) {
				graphElementRemoved(node);
			}
		}

		nodes = new LinkedHashSet<PatternTreeNode>();
		arcs = new LinkedHashSet<PatternTreeEdge<PatternTreeNode, PatternTreeNode>>();
		//add nodes
		sizeAlphabetSetMap = new HashMap<Integer, Set<PatternTreeNode>>();
		Set<PatternTreeNode> sizeAlphabetSet;
		//	Integer alphabetCount = 0;
		//set the vertices of the Pattern in the graph
		for (Set<String> alphabet : filterPatternAlphabetSet) {
			PatternTreeNode treeNode = addNode(alphabet);
			if (sizeAlphabetSetMap.containsKey(alphabet.size())) {
				sizeAlphabetSet = sizeAlphabetSetMap.get(alphabet.size());
			} else {
				sizeAlphabetSet = new HashSet<PatternTreeNode>();
			}
			sizeAlphabetSet.add(treeNode);
			sizeAlphabetSetMap.put(alphabet.size(), sizeAlphabetSet);
		}

		// make an array of the different size of the filter pattern alphabet
		sortedSizeArray = new int[sizeAlphabetSetMap.size()];
		int index = 0;
		for (Integer size : sizeAlphabetSetMap.keySet()) {
			sortedSizeArray[index++] = size;
		}
		Arrays.sort(sortedSizeArray);

		//set the vertices of each single activity which is a part of one filter pattern,
		//but not form a pattern by itself

		/*
		 * Set<PatternTreeNode> largerPatternNodeSet; Set<String>
		 * activitiesInPattern; Set<PatternTreeNode> artificialPatternNodeSet =
		 * new HashSet<PatternTreeNode>(); for (Integer i :
		 * sizeAlphabetSetMap.keySet()) { if (i > 1) { largerPatternNodeSet =
		 * sizeAlphabetSetMap.get(i); for (PatternTreeNode node :
		 * largerPatternNodeSet) { activitiesInPattern =
		 * PatternTreeUtil.getActivityLabelsInPattern(node, encodingLength);
		 * Set<PatternTreeNode> predecessors = node.getPredecessors();
		 * Set<String> labelsOfPreNodes = new HashSet<String>(); for
		 * (PatternTreeNode existPreNode : predecessors) { String
		 * precessorLabel; precessorLabel = existPreNode.getLabel(); Set<String>
		 * activityNameInPrecessorNode =
		 * PatternTreeUtil.getOriginalActivityNameInPattern( precessorLabel,
		 * encodingLength); for (String activityName :
		 * activityNameInPrecessorNode) { labelsOfPreNodes.add(activityName); }
		 * } if (labelsOfPreNodes.size() < activitiesInPattern.size()) { //get
		 * every predecessor node's name
		 * activitiesInPattern.removeAll(labelsOfPreNodes); for (String
		 * activityName : activitiesInPattern) { String labelOfNodeToAdd = "[" +
		 * activityName + "]"; TreeSet<String> labelAlphabet = new
		 * TreeSet<String>(); labelAlphabet.add(activityName); PatternTreeNode
		 * originalNode = containNode(labelOfNodeToAdd); if (originalNode ==
		 * null) { PatternTreeNode artificialNode = addNode(labelAlphabet);
		 * artificialPatternNodeSet.add(artificialNode); } } } } } } //adjust
		 * the value of sizeAlphabetSetMap to add arc for the artificial pattern
		 * nodes for (PatternTreeNode artificialNode : artificialPatternNodeSet)
		 * {
		 * 
		 * sizeAlphabetSetMap.get(1).add(artificialNode); }
		 */

		//set the vertices of not being used single activity(not pattern)
		/*
		 * for (Set<String> alphabet : notUsedSingleActivitySet) {
		 * addNode(alphabet); }
		 */

	}

	public void addNotUsedSingleActivityNodes(Set<Set<String>> notUsedSingleActivitySet) {
		//set the vertices of not being used single activity(not pattern)
		for (Set<String> alphabet : notUsedSingleActivitySet) {
			addNode(alphabet);
		}
	}
	/*
	 * Set edges between the pattern alphabet nodes
	 */
	public void setBasicEdges(Set<Set<String>> filterPatternAlphabetSet) {
		SimpleDirectedGraph<Set<String>, DefaultEdge> hasseDiagram = ptnAbsFrm.getHasseDiagram(filterPatternAlphabetSet);
		int maxStepSize = sortedSizeArray.length - 1;
		int lowerLayerAlphabetSize, upperLayerAlphabetSize;
		Set<PatternTreeNode> lowerLayerPatternNodeSet, upperLayerPatternNodeSet;
		Set<String> lowerLayerPatternLabel, upperLayerPatternLabel;
		for (int step = 1; step <= maxStepSize; step++) {
			System.out.println("Step : " + step);
			//for (int i = 0; i < sortedSizeArray.length - step; i++) {
			for (int i = sortedSizeArray.length - 1; i >= step; i--) {
				System.out.println("\tLayer " + i + ", " + (i - step));
				upperLayerAlphabetSize = sortedSizeArray[i];
				lowerLayerAlphabetSize = sortedSizeArray[i - step];

				lowerLayerPatternNodeSet = sizeAlphabetSetMap.get(lowerLayerAlphabetSize);
				upperLayerPatternNodeSet = sizeAlphabetSetMap.get(upperLayerAlphabetSize);
				for (PatternTreeNode alphabetI : lowerLayerPatternNodeSet) {
					for (PatternTreeNode alphabetJ : upperLayerPatternNodeSet) {
						lowerLayerPatternLabel = PatternTreeUtil.getActivityLabelsInPattern(alphabetI, encodingLength);
						upperLayerPatternLabel = PatternTreeUtil.getActivityLabelsInPattern(alphabetJ, encodingLength);
						if (hasseDiagram.containsEdge(lowerLayerPatternLabel, upperLayerPatternLabel)) {
							addArc(alphabetI, alphabetJ);
						}
					}
				}
			}
		}

	}

	public void setExtraEdges() {
		Set<PatternTreeNode> largerPatternNodeSet;
		Set<String> activitiesInPattern;
		for (Integer i : sizeAlphabetSetMap.keySet()) {
			if (i > 1) {
				largerPatternNodeSet = sizeAlphabetSetMap.get(i);
				for (PatternTreeNode node : largerPatternNodeSet) {
					activitiesInPattern = FuzzyMapUtil.getActivityLabelsInPattern(node, encodingLength);
					Set<PatternTreeNode> predecessors = node.getPredecessors();
					Set<String> labelsOfPreNodes = new HashSet<String>();
					for (PatternTreeNode existPreNode : predecessors) {
						String precessorLabel;
						precessorLabel = existPreNode.getLabel();
						Set<String> activityNameInPrecessorNode = FuzzyMapUtil.getOriginalActivityNameInPattern(
								precessorLabel, encodingLength);
						for (String activityName : activityNameInPrecessorNode) {
							labelsOfPreNodes.add(activityName);
						}
					}
					if (labelsOfPreNodes.size() < activitiesInPattern.size()) {
						//get every predecessor node's name
						activitiesInPattern.removeAll(labelsOfPreNodes);
						for (String activityName : activitiesInPattern) {
							String labelOfNodeToAdd = "[" + activityName + "]";
							TreeSet<String> labelAlphabet = new TreeSet<String>();
							labelAlphabet.add(activityName);
							PatternTreeNode originalNode = containNode(labelOfNodeToAdd);
							if (originalNode != null) {
								addArc(originalNode, node);
							} else {
								PatternTreeNode sourceNode = addNode(labelAlphabet);
								addArc(sourceNode, node);
							}
						}
					}
				}
			}

		}
	}

	/*
	 * public void setEdges() { int maxStepSize = sortedSizeArray.length - 1;
	 * int lowerLayerAlphabetSize, upperLayerAlphabetSize; Set<PatternTreeNode>
	 * lowerLayerPatternNodeSet, upperLayerPatternNodeSet; Set<String>
	 * lowerLayerPatternLabel, upperLayerPatternLabel; for (int step = 1; step
	 * <= maxStepSize; step++) { System.out.println("Step : " + step); //for
	 * (int i = 0; i < sortedSizeArray.length - step; i++) { for (int i =
	 * sortedSizeArray.length - 1; i >= step; i--) {
	 * System.out.println("\tLayer " + i + ", " + (i - step));
	 * upperLayerAlphabetSize = sortedSizeArray[i]; lowerLayerAlphabetSize =
	 * sortedSizeArray[i - step];
	 * 
	 * lowerLayerPatternNodeSet =
	 * sizeAlphabetSetMap.get(lowerLayerAlphabetSize); upperLayerPatternNodeSet
	 * = sizeAlphabetSetMap.get(upperLayerAlphabetSize); for (PatternTreeNode
	 * alphabetI : lowerLayerPatternNodeSet) { for (PatternTreeNode alphabetJ :
	 * upperLayerPatternNodeSet) { lowerLayerPatternLabel =
	 * PatternTreeUtil.getActivityLabelsInPattern(alphabetI, encodingLength);
	 * upperLayerPatternLabel =
	 * PatternTreeUtil.getActivityLabelsInPattern(alphabetJ, encodingLength); if
	 * (upperLayerPatternLabel.containsAll(lowerLayerPatternLabel)) {
	 * ShortestPathInfo<PatternTreeNode, PatternTreeEdge<? extends
	 * PatternTreeNode, ? extends PatternTreeNode>> shortestPathInfo =
	 * ShortestPathFactory .calculateAllShortestDistanceDijkstra(this); if
	 * (shortestPathInfo.getShortestPathLength(alphabetI, alphabetJ) < 0) {
	 * addArc(alphabetI, alphabetJ); } } } } } } }
	 */

	@Override
	protected PatternTreeImpl getEmptyClone() {
		return new PatternTreeImpl(getLabel());
	}

	public Map<? extends DirectedGraphElement, ? extends DirectedGraphElement> cloneFrom(
			DirectedGraph<PatternTreeNode, PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> graph) {
		HashMap<DirectedGraphElement, DirectedGraphElement> mapping = new HashMap<DirectedGraphElement, DirectedGraphElement>();

		for (PatternTreeNode a : graph.getNodes()) {
			mapping.put(a, addNode(a.getLabelAlphabet()));
		}

		getAttributeMap().clear();
		AttributeMap map = graph.getAttributeMap();
		for (String key : map.keySet()) {
			getAttributeMap().put(key, map.get(key));
		}
		return mapping;
	}

	public PatternTreeNode containNode(String label) {
		PatternTreeNode containedNode = null;
		for (PatternTreeNode node : nodes) {
			if (node.getLabel().equals(label)) {
				containedNode = node;
				break;
			}
		}
		return containedNode;
	}

	/**
	 * NODE
	 */
	public synchronized PatternTreeNode addNode(Set<String> label) {
		PatternTreeNode node = new PatternTreeNodeElement(label, this);
		Double alphabetTotalCount;
		Double alphabetStandaloneCount;
		Double alphabetConservednessCount;
		//set node metrics
		if (actualNonOverlapAlphabetCountMap.containsKey(label)) {
			// this is a pattern node
			alphabetTotalCount = new Double(actualNonOverlapAlphabetCountMap.get(label));
		} else {
			//this is a single activity node
			alphabetTotalCount = new Double(encodedActivityCountMap.get(label.iterator().next()));
		}

		if (patternIsolationNonOverlapAlphabetCountMap.containsKey(label)) {
			alphabetStandaloneCount = new Double(patternIsolationNonOverlapAlphabetCountMap.get(label));
		} else {
			alphabetStandaloneCount = new Double(0.0);
		}

		if (actualNonOverlapAlphabetConservednessMetricMap.containsKey(label)) {
			alphabetConservednessCount = new Double(actualNonOverlapAlphabetConservednessMetricMap.get(label));
		} else {
			alphabetConservednessCount = 100.0;
		}
		node.setAlphabetTotalCount(alphabetTotalCount);
		node.setAlphabetStandaloneCount(alphabetStandaloneCount);
		node.setAlphabetConservednessCount(alphabetConservednessCount);
		nodes.add(node);
		graphElementAdded(node);
		return node;
	}

	public synchronized PatternTreeNode removeNode(PatternTreeNode PatternTreeNode) {
		removeSurroundingEdges(PatternTreeNode);
		return removeNodeFromCollection(nodes, PatternTreeNode);
	}

	public void removeNode(DirectedGraphNode cell) {
		if (cell instanceof PatternTreeNode) {
			removeNode((PatternTreeNode) cell);
		}
	}

	public synchronized Set<PatternTreeNode> getNodes() {
		Set<PatternTreeNode> newNodes = new HashSet<PatternTreeNode>();
		newNodes.addAll(nodes);
		return newNodes;
	}

	/**
	 * ARC
	 */
	public synchronized PatternTreeEdge<PatternTreeNode, PatternTreeNode> addArc(PatternTreeNode source,
			PatternTreeNode target) {
		return addArc(source, target, null);
	}

	public synchronized PatternTreeEdge<PatternTreeNode, PatternTreeNode> addArc(PatternTreeNode source,
			PatternTreeNode target, String label) {
		checkAddEdge(source, target);
		PatternTreeArcElement a = new PatternTreeArcElement(source, target);
		if (arcs.add(a)) {
			graphElementAdded(a);
			return a;
		} else {
			for (PatternTreeEdge<PatternTreeNode, PatternTreeNode> existing : arcs) {
				if (existing.equals(a)) {
					if (label != null) {
						existing.getAttributeMap().put(AttributeMap.LABEL, label);
						graphElementChanged(existing);
					}
					return existing;
				}
			}
		}
		assert (false);
		return null;
	}

	public synchronized PatternTreeEdge<PatternTreeNode, PatternTreeNode> removeArc(PatternTreeNode source,
			PatternTreeNode target) {
		return removeFromEdges(source, target, arcs);
	}

	public PatternTreeEdge<PatternTreeNode, PatternTreeNode> removeArc(
			PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode> edge) {
		return removeFromEdges(edge.getSource(), edge.getTarget(), arcs);
	}

	@SuppressWarnings("unchecked")
	public void removeEdge(DirectedGraphEdge edge) {
		if (edge instanceof PatternTreeArcElement) {
			arcs.remove(edge);
		} else {
			assert (false);
		}
		graphElementRemoved(edge);
	}

	public synchronized PatternTreeEdge<PatternTreeNode, PatternTreeNode> getArc(PatternTreeNode source,
			PatternTreeNode target) {
		Collection<PatternTreeEdge<PatternTreeNode, PatternTreeNode>> set = getEdges(source, target, arcs);
		return (set.isEmpty() ? null : set.iterator().next());
	}

	public synchronized Set<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> getEdges() {
		Set<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> edges = new HashSet<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>>();
		edges.addAll(arcs);
		return edges;
	}

	public synchronized Set<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> getInEdges(
			PatternTreeNode targetNode) {
		Set<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> inEdges = new HashSet<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>>();
		for (PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode> edge : arcs) {
			if (edge.getTarget().equals(targetNode)) {
				inEdges.add(edge);
			}
		}
		return inEdges;
	}

	public synchronized Set<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> getOutEdges(
			PatternTreeNode sourceNode) {
		Set<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>> outEdges = new HashSet<PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode>>();
		for (PatternTreeEdge<? extends PatternTreeNode, ? extends PatternTreeNode> edge : arcs) {
			if (edge.getSource().equals(sourceNode)) {
				outEdges.add(edge);
			}
		}
		return outEdges;
	}

	public Map<String, Integer> getEncodedActivityCountMap() {
		return encodedActivityCountMap;
	}

	public void setEncodedActivityCountMap(Map<String, Integer> encodedActivityCountMap) {
		this.encodedActivityCountMap = encodedActivityCountMap;
	}

	public Map<TreeSet<String>, Integer> getActualNonOverlapAlphabetCountMap() {
		return actualNonOverlapAlphabetCountMap;
	}

	public void setActualNonOverlapAlphabetCountMap(Map<TreeSet<String>, Integer> actualNonOverlapAlphabetCountMap) {
		this.actualNonOverlapAlphabetCountMap = actualNonOverlapAlphabetCountMap;
	}

	public Map<TreeSet<String>, Integer> getPatternIsolationNonOverlapAlphabetCountMap() {
		return patternIsolationNonOverlapAlphabetCountMap;
	}

	public void setPatternIsolationNonOverlapAlphabetCountMap(
			Map<TreeSet<String>, Integer> patternIsolationNonOverlapAlphabetCountMap) {
		this.patternIsolationNonOverlapAlphabetCountMap = patternIsolationNonOverlapAlphabetCountMap;
	}

	public Map<TreeSet<String>, Integer> getActualNonOverlapAlphabetConservednessMetricMap() {
		return actualNonOverlapAlphabetConservednessMetricMap;
	}

	public void setActualNonOverlapAlphabetConservednessMetricMap(
			Map<TreeSet<String>, Integer> actualNonOverlapAlphabetConservednessMetricMap) {
		this.actualNonOverlapAlphabetConservednessMetricMap = actualNonOverlapAlphabetConservednessMetricMap;
	}

}
