package org.processmining.plugins.ptnBasedPreprocess.util;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.JInternalFrame;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.jgraph.graph.DefaultEdge;
import org.jgrapht.graph.SimpleDirectedGraph;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.abstractions.AbstractionSetTheory;
import org.processmining.plugins.abstractions.PatternAbstractionFrame;

public class PatternAbstractionFrameUtil extends PatternAbstractionFrame {
	


	/**
	 * 
	 */
	private static final long serialVersionUID = -3632157231323021754L;
	private Map<TreeSet<String>, TreeSet<String>> actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap;
	private Map<TreeSet<String>, Integer> actualNonOverlapAlphabetCountMap;
	private Map<TreeSet<String>, Integer> actualNonOverlapAlphabetInstanceCountMap;
	private Map<TreeSet<String>, Integer> actualNonOverlapAlphabetConservednessMetricMap;
	private boolean isShorterPatternsPreferred = true;
//	private Map<TreeSet<String>, Integer> actualStandAloneAlphabetCountMap;
	//private boolean ignoreDuplicates = true;
	//private Map<String, Integer> encodedActivityCountMap;
	

	public PatternAbstractionFrameUtil(UIPluginContext context, XLog log) {
		super(context,log);
		actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap = getActualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap();
		actualNonOverlapAlphabetCountMap = getActualNonOverlapAlphabetCountMap();
		actualNonOverlapAlphabetInstanceCountMap = getActualNonOverlapAlphabetInstanceCountMap();
		actualNonOverlapAlphabetConservednessMetricMap = getActualNonOverlapAlphabetConservednessMetricMap();
	//	actualStandAloneAlphabetCountMap = getActualStandAloneAlphabetCountMap();
		
	}
	//ljf add 
	//find all the repeatPattern in a given log considering only base patterns and duplicate traces
	public void findRepeatPatternsInLog() {
		encodeLog();
		//basePatternCheckBox.setSelected(false);
		setBasePatternCheckBox(false);
		findRepeatPatterns();
		setActualPattern();
		setIgnoreDuplicates(false);
		findPatternCount();
		findNonOverlapPatternCount();
	}

	

	//prepare the initial parameters which are suitable for filter the patterns  
	public int[] prepareInitialFilterParameters() {
		int[] filterParas = new int[3];
		int sumOfAlphabetCount, sumOfAlphabetInstanceCount, sumOfAlphabetConserveness;
		int avgFreq, avgInstanceCount, avgConserveness = 0;

		Set<Set<String>> freqFilterAlphabetSet = new HashSet<Set<String>>();
		Set<Set<String>> instanceCountFilterAlphabetSet = new HashSet<Set<String>>();
		Set<Set<String>> conservenessMetricFilterAlphabetSet = new HashSet<Set<String>>();
		//calculate the average pattern alphabet frequency
		sumOfAlphabetCount = 0;
		for (Set<String> alphabet : actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap.keySet()) {
			sumOfAlphabetCount += actualNonOverlapAlphabetCountMap.get(alphabet);
		}
		avgFreq = sumOfAlphabetCount / actualNonOverlapAlphabetCountMap.size();
		//filter the patterns according to the average frequency
		for (Set<String> alphabet : actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap.keySet()) {
			if (actualNonOverlapAlphabetCountMap.get(alphabet) >= avgFreq) {
				freqFilterAlphabetSet.add(alphabet);
			}
		}
		/*
		 * calculate the average pattern instance count based on the filtered
		 * patterns whose frequency are larger than the average frequency
		 */
		sumOfAlphabetInstanceCount = 0;
		for (Set<String> alphabet : freqFilterAlphabetSet) {
			sumOfAlphabetInstanceCount += actualNonOverlapAlphabetInstanceCountMap.get(alphabet);
		}
		avgInstanceCount = sumOfAlphabetInstanceCount / freqFilterAlphabetSet.size();
		//filter the patterns according to the average instane count
		for (Set<String> alphabet : freqFilterAlphabetSet) {
			if (actualNonOverlapAlphabetInstanceCountMap.get(alphabet) >= avgInstanceCount) {
				instanceCountFilterAlphabetSet.add(alphabet);
			}
		}

		/*
		 * calculate the average pattern conserveness based on the filtered
		 * patterns whose frequency are larger than the average frequency of the
		 * original patterns and instance count are larger than the average
		 * instance account in the filtered patterns
		 */
		sumOfAlphabetConserveness = 0;
		for (Set<String> alphabet : instanceCountFilterAlphabetSet) {
			sumOfAlphabetConserveness += actualNonOverlapAlphabetConservednessMetricMap.get(alphabet);
		}
		avgConserveness = sumOfAlphabetConserveness / instanceCountFilterAlphabetSet.size();
		for (Set<String> alphabet : instanceCountFilterAlphabetSet) {
			if (actualNonOverlapAlphabetConservednessMetricMap.get(alphabet) >= avgConserveness) {
				conservenessMetricFilterAlphabetSet.add(alphabet);
			}
		}
		//Set the initial filter parameters
		filterParas[0] = avgFreq;
		filterParas[1] = avgInstanceCount;
		filterParas[2] = avgConserveness;
		System.out.println("The avgFreq is " + filterParas[0]);
		System.out.println("The avgInstanceCount is " + filterParas[1]);
		System.out.println("The avgInstanceCount is " + filterParas[2]);
		return filterParas;
	}

	/*
	 * Filter the original patterns according to the speicified parameters
	 */
	public Set<Set<String>> getFilterPatternAlphabetSet(int freqThreshold, int instanceCountThreshold,
			int conservenessThreshold, int alphabetSize) {
		Set<Set<String>> filterPatternAlphabetSet = new HashSet<Set<String>>();
		for (Set<String> alphabet : actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap.keySet()) {
			if ((actualNonOverlapAlphabetCountMap.get(alphabet) > freqThreshold)
					&& (actualNonOverlapAlphabetInstanceCountMap.get(alphabet) > instanceCountThreshold)
					&& (actualNonOverlapAlphabetConservednessMetricMap.get(alphabet) > conservenessThreshold)
					&& (alphabet.size() > alphabetSize)) {
				filterPatternAlphabetSet.add(alphabet);
			}
		}
		return filterPatternAlphabetSet;
	}

	/*
	 * get the HasseDiagram based on the filtered patterns
	 */
	public SimpleDirectedGraph<Set<String>, DefaultEdge> getHasseDiagram(Set<Set<String>> filterPatternAlphabetSet) {
		SimpleDirectedGraph<Set<String>, DefaultEdge> hasseDiagram;
		AbstractionSetTheory ast = new AbstractionSetTheory(filterPatternAlphabetSet);
		hasseDiagram = ast.getHasseDiagram();
		return hasseDiagram;
	}

	/*
	 * get the map of the filtered pattern and its sequence set
	 */
	public Map<Set<String>, TreeSet<String>> getFilterPatternAlphabetSequenceSetMap(
			Set<Set<String>> filterPatternAlphabetSet) {
		Map<Set<String>, TreeSet<String>> filterPatternAlphabetSequenceSetMap = new HashMap<Set<String>, TreeSet<String>>();
		for (Set<String> alphabet : filterPatternAlphabetSet) {
			//			filterPatternAlphabetSequenceSetMap.put(alphabet, basePatternAlphabetEquivalenceClassPatternSetMap
			//					.get(alphabet));
			filterPatternAlphabetSequenceSetMap.put(alphabet,
					actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap.get(alphabet));
		}
		return filterPatternAlphabetSequenceSetMap;
	}

	/*
	 * get the map of the maximal element and its sequence set. Note that there
	 * could be the same map as the one in the map of the filter Pattern and its
	 * sequence set
	 */
	public Map<Set<String>, TreeSet<String>> getAbstractionAlphabetSequenceSetMap(
			Set<Set<String>> filterPatternAlphabetSet) {
		Map<Set<String>, TreeSet<String>> abstractionAlphabetSequenceSetMap = new HashMap<Set<String>, TreeSet<String>>();
		AbstractionSetTheory ast = new AbstractionSetTheory(filterPatternAlphabetSet);
		for (Set<String> maximalElement : ast.getMaximalElements()) {
			//			abstractionAlphabetSequenceSetMap.put(maximalElement, basePatternAlphabetEquivalenceClassPatternSetMap
			//					.get(maximalElement));
			//ljf changed to use basepatterns
			abstractionAlphabetSequenceSetMap.put(maximalElement,
					actualNonOverlapPatternAlphabetEquivalenceClassPatternSetMap.get(maximalElement));
		}
		return abstractionAlphabetSequenceSetMap;
	}

	//This method just considers the first repeat matching a position 
	public void findNonoverlapPatternCountAcrossAllPatterns(boolean isShorterPatternsPreferred) {
		boolean originalValue = this.isShorterPatternsPreferred;
		this.isShorterPatternsPreferred = isShorterPatternsPreferred;
		findNonoverlapPatternCountAcrossAllPatterns();
		this.isShorterPatternsPreferred = originalValue;
	}

	//get the count of each standalone pattern
//	public Map<TreeSet<String>, Integer> getActualStandAloneAlphabetCountMap() {
//		return actualStandAloneAlphabetCountMap;
//	}
//
//	//
//	public Map<TreeSet<String>, Integer> getActualNonOverlapAlphabetConservednessMetricMap() {
//		return actualNonOverlapAlphabetConservednessMetricMap;
//	}

//	public void setIgnoreDuplicates(boolean isIgnoreDuplicates) {
//		ignoreDuplicates = isIgnoreDuplicates;
//	}

	public Set<Set<String>> getSingleActivityNotUsedInPatterns(Set<Set<String>> filterPatternAlphabetSet,
			Map<String, Integer> currentEncodedActivityCountMap) {
		Set<Set<String>> singleNodes = new HashSet<Set<String>>();
		Set<String> charSets = new HashSet<String>();

		for (String actChar : currentEncodedActivityCountMap.keySet()) {
			charSets.add(actChar);
		}
		//get the nodes already used in a pattern of the filtered pattern alphabet set
		Set<String> usedCharSets = new HashSet<String>();
		for (Set<String> alphabet : filterPatternAlphabetSet) {
			usedCharSets.addAll(alphabet);
		}
		charSets.removeAll(usedCharSets);
		for (String notUsedChar : charSets) {
			Set<String> singleAlphabet = new HashSet<String>();
			singleAlphabet.add(notUsedChar);
			singleNodes.add(singleAlphabet);
		}
		return singleNodes;
	}
//


	//ljf addition end

}
