package org.processmining.plugins.ptnBasedPreprocess.util;

import java.io.File;
import java.util.Collection;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.PluginExecutionResult;
import org.processmining.framework.plugin.PluginParameterBinding;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.events.Logger.MessageLevel;
import org.processmining.framework.util.Pair;
import org.processmining.plugins.log.logabstraction.LogRelations;

public class PatternBasedPreprocessUtil {
	private static String[] illegalCharSetsInFileName = { "\\", "/", ":", "?", "\"", "<", ">", "|" };
	private static int numOfIllegalChars = illegalCharSetsInFileName.length;

	static public String getLegalFileName(String illegalFileName) {
	//	System.out.println("!!!The input file name is " + illegalFileName);
		for (int i = 0; i < numOfIllegalChars; i++) {

			illegalFileName = illegalFileName.replace(illegalCharSetsInFileName[i], " ");
		}

		String leagalFileName = illegalFileName;
//		System.out.println("!!!!The out  file name is " + illegalFileName);
		return leagalFileName;
	}

	/*
	 * get the causal relations in the specified log
	 */
	static public LogRelations getLogRelations(PluginContext context, XLog log) throws CancellationException,
			InterruptedException, ExecutionException {
		XLogInfo summary = XLogInfoFactory.createLogInfo(log);
		Collection<Pair<Integer, PluginParameterBinding>> plugins = context.getPluginManager().find(Plugin.class,
				LogRelations.class, context.getPluginContextType(), true, false, false, XLog.class, summary.getClass());

		if (plugins.isEmpty()) {
			context.log("No plugin found to create log relations, please specify relations manually",
					MessageLevel.ERROR);
			return null;
		}
		// Let's just take the first available plugin for the job of constructing log abstractions
		Pair<Integer, PluginParameterBinding> plugin = plugins.iterator().next();

		// Now, the binding can be executed on the log and the summary
		// FIrst, we instantiate a new context for this plugin, which is a child context of the current context. 
		PluginContext c2 = context.createChildContext("Log Relation Constructor");

		// Let's notify our lifecyclelisteners about the fact that we created a new context. this is
		// optional, but if this is not done, then the user interface doesn't show it (if there is a UI).
		context.getPluginLifeCycleEventListeners().firePluginCreated(c2);

		// At this point, we execute the binding to get the LogRelations. For this, we call the invoke method
		// on the PluginParameterBinding stored in the plugin variable. The return type is LogRelations.class and
		// as input we give the new context c2, the log and the summary. Note that the plugin might return mulitple
		// objects, hence we extract the object with number x, where x is stored as the first element of the plugin
		// variable.

		PluginExecutionResult pluginResult = plugin.getSecond().invoke(c2, log, summary);
		pluginResult.synchronize();
		LogRelations relations = pluginResult.<LogRelations>getResult(plugin.getFirst());
		return relations;
	}

	/*
	 * Delete the specified directory and all the files in it
	 */
	static public boolean deleteDirectory(File path) {
		if (path.exists()) {
			File[] files = path.listFiles();
			for (int i = 0; i < files.length; i++) {
				if (files[i].isDirectory()) {
					deleteDirectory(files[i]);
				} else {
					files[i].delete();
				}
			}
		}
		return (path.delete());
	}

}
